@@ -9,7 +9,7 @@
   The configuration file is now stored in an ASCII format to allow the user
   to edit the file manually.
 */
-const char Configuration_rcsid[] = "Hatari $Id: configuration.c,v 1.65 2007/02/27 20:53:52 eerot Exp $";
+const char Configuration_rcsid[] = "Hatari $Id: configuration.c,v 1.84 2008/03/11 20:11:08 eerot Exp $";
 
 #include <SDL_keysym.h>
 
@@ -21,10 +21,10 @@ const char Configuration_rcsid[] = "Hatari $Id: configuration.c,v 1.65 2007/02/2
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
+#include "paths.h"
 #include "screen.h"
 #include "vdi.h"
 #include "video.h"
-#include "uae-cpu/hatari-glue.h"
 
 
 BOOL bFirstTimeInstall = FALSE;             /* Has been run before? Used to set default joysticks etc... */
@@ -38,6 +38,7 @@ static const struct Config_Tag configs_Log[] =
 	{ "sLogFileName", String_Tag, ConfigureParams.Log.sLogFileName },
 	{ "nTextLogLevel", Int_Tag, &ConfigureParams.Log.nTextLogLevel },
 	{ "nAlertDlgLogLevel", Int_Tag, &ConfigureParams.Log.nAlertDlgLogLevel },
+	{ "bConfirmQuit", Bool_Tag, &ConfigureParams.Log.bConfirmQuit },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -45,17 +46,25 @@ static const struct Config_Tag configs_Log[] =
 /* Used to load/save screen options */
 static const struct Config_Tag configs_Screen[] =
 {
-	{ "bFullScreen", Bool_Tag, &ConfigureParams.Screen.bFullScreen },
-	{ "FrameSkips", Int_Tag, &ConfigureParams.Screen.FrameSkips },
-	{ "bAllowOverscan", Bool_Tag, &ConfigureParams.Screen.bAllowOverscan },
-	{ "bForce8Bpp", Bool_Tag, &ConfigureParams.Screen.bForce8Bpp },
-	{ "bZoomLowRes", Bool_Tag, &ConfigureParams.Screen.bZoomLowRes },
 	{ "MonitorType", Int_Tag, &ConfigureParams.Screen.MonitorType },
+	{ "FrameSkips", Int_Tag, &ConfigureParams.Screen.FrameSkips },
+	{ "bFullScreen", Bool_Tag, &ConfigureParams.Screen.bFullScreen },
+	{ "bAllowOverscan", Bool_Tag, &ConfigureParams.Screen.bAllowOverscan },
+	{ "nSpec512Threshold", Int_Tag, &ConfigureParams.Screen.nSpec512Threshold },
+	{ "nForceBpp", Int_Tag, &ConfigureParams.Screen.nForceBpp },
+	{ "bZoomLowRes", Bool_Tag, &ConfigureParams.Screen.bZoomLowRes },
 	{ "bUseExtVdiResolutions", Bool_Tag, &ConfigureParams.Screen.bUseExtVdiResolutions },
-	{ "nVdiResolution", Int_Tag, &ConfigureParams.Screen.nVdiResolution },
+	{ "nVdiWidth", Int_Tag, &ConfigureParams.Screen.nVdiWidth },
+	{ "nVdiHeight", Int_Tag, &ConfigureParams.Screen.nVdiHeight },
 	{ "nVdiColors", Int_Tag, &ConfigureParams.Screen.nVdiColors },
 	{ "bCaptureChange", Bool_Tag, &ConfigureParams.Screen.bCaptureChange },
 	{ "nFramesPerSecond", Int_Tag, &ConfigureParams.Screen.nFramesPerSecond },
+	{ "nWindowBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsLeft },
+	{ "nWindowBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsRight },
+	{ "nWindowBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsBottom },
+	{ "nFullScreenBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsLeft },
+	{ "nFullScreenBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsRight },
+	{ "nFullScreenBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsBottom },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -168,6 +177,8 @@ static const struct Config_Tag configs_ShortCutWithMod[] =
 	{ "keyRecSound",   Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_RECSOUND] },
 	{ "keySound",      Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SOUND] },
 	{ "keyQuit",       Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_QUIT] },
+	{ "keyLoadMem",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_LOADMEM] },
+	{ "keySaveMem",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SAVEMEM] },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -187,6 +198,8 @@ static const struct Config_Tag configs_ShortCutWithoutMod[] =
 	{ "keyRecSound",   Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_RECSOUND] },
 	{ "keySound",      Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SOUND] },
 	{ "keyQuit",       Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_QUIT] },
+	{ "keyLoadMem",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_LOADMEM] },
+	{ "keySaveMem",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SAVEMEM] },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -204,7 +217,9 @@ static const struct Config_Tag configs_Sound[] =
 static const struct Config_Tag configs_Memory[] =
 {
 	{ "nMemorySize", Int_Tag, &ConfigureParams.Memory.nMemorySize },
+	{ "bAutoSave", Bool_Tag, &ConfigureParams.Memory.bAutoSave },
 	{ "szMemoryCaptureFileName", String_Tag, ConfigureParams.Memory.szMemoryCaptureFileName },
+	{ "szAutoSaveFileName", String_Tag, ConfigureParams.Memory.szAutoSaveFileName },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -269,7 +284,6 @@ static const struct Config_Tag configs_Midi[] =
 /* Used to load/save system options */
 static const struct Config_Tag configs_System[] =
 {
-	{ "nMinMaxSpeed", Int_Tag, &ConfigureParams.System.nMinMaxSpeed },
 	{ "nCpuLevel", Int_Tag, &ConfigureParams.System.nCpuLevel },
 	{ "nCpuFreq", Int_Tag, &ConfigureParams.System.nCpuFreq },
 	{ "bCompatibleCpu", Bool_Tag, &ConfigureParams.System.bCompatibleCpu },
@@ -279,6 +293,7 @@ static const struct Config_Tag configs_System[] =
 	{ "bRealTimeClock", Bool_Tag, &ConfigureParams.System.bRealTimeClock },
 	{ "bPatchTimerD", Bool_Tag, &ConfigureParams.System.bPatchTimerD },
 	{ "bSlowFDC", Bool_Tag, &ConfigureParams.System.bSlowFDC },
+	{ "nMinMaxSpeed", Int_Tag, &ConfigureParams.System.nMinMaxSpeed },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -290,7 +305,11 @@ static const struct Config_Tag configs_System[] =
 void Configuration_SetDefault(void)
 {
 	int i;
-	char *homeDir;
+	const char *psHomeDir;
+	const char *psWorkingDir;
+
+	psHomeDir = Paths_GetHatariHome();
+	psWorkingDir = Paths_GetWorkingDir();
 
 	/* Assume first-time install */
 	bFirstTimeInstall = TRUE;
@@ -302,11 +321,12 @@ void Configuration_SetDefault(void)
 	strcpy(ConfigureParams.Log.sLogFileName, "stderr");
 	ConfigureParams.Log.nTextLogLevel = LOG_INFO;
 	ConfigureParams.Log.nAlertDlgLogLevel = LOG_INFO;
+	ConfigureParams.Log.bConfirmQuit = TRUE;
 
 	/* Set defaults for floppy disk images */
 	ConfigureParams.DiskImage.bAutoInsertDiskB = TRUE;
 	ConfigureParams.DiskImage.nWriteProtection = WRITEPROT_OFF;
-	strcpy(ConfigureParams.DiskImage.szDiskImageDirectory, szWorkingDir);
+	strcpy(ConfigureParams.DiskImage.szDiskImageDirectory, psWorkingDir);
 	File_AddSlashToEndFileName(ConfigureParams.DiskImage.szDiskImageDirectory);
 
 	/* Set defaults for hard disks */
@@ -315,13 +335,13 @@ void Configuration_SetDefault(void)
 	ConfigureParams.HardDisk.bUseHardDiskDirectories = FALSE;
 	for (i=0; i<MAX_HARDDRIVES; i++)
 	{
-		strcpy(ConfigureParams.HardDisk.szHardDiskDirectories[i], szWorkingDir);
+		strcpy(ConfigureParams.HardDisk.szHardDiskDirectories[i], psWorkingDir);
 		File_CleanFileName(ConfigureParams.HardDisk.szHardDiskDirectories[i]);
 	}
 	ConfigureParams.HardDisk.bUseHardDiskImage = FALSE;
-	strcpy(ConfigureParams.HardDisk.szHardDiskImage, szWorkingDir);
+	strcpy(ConfigureParams.HardDisk.szHardDiskImage, psWorkingDir);
 	ConfigureParams.HardDisk.bUseIdeHardDiskImage = FALSE;
-	strcpy(ConfigureParams.HardDisk.szIdeHardDiskImage, szWorkingDir);
+	strcpy(ConfigureParams.HardDisk.szIdeHardDiskImage, psWorkingDir);
 
 	/* Set defaults for Joysticks */
 	for (i = 0; i < JOYSTICK_COUNT; i++)
@@ -365,12 +385,17 @@ void Configuration_SetDefault(void)
 	
 	/* Set defaults for Memory */
 	ConfigureParams.Memory.nMemorySize = 1;     /* 1 MiB */
-	sprintf(ConfigureParams.Memory.szMemoryCaptureFileName, "%s%chatari.sav", szWorkingDir,PATHSEP);
+	ConfigureParams.Memory.bAutoSave = FALSE;
+	sprintf(ConfigureParams.Memory.szMemoryCaptureFileName, "%s%chatari.sav",
+	        psHomeDir, PATHSEP);
+	sprintf(ConfigureParams.Memory.szAutoSaveFileName, "%s%cauto.sav",
+	        psHomeDir, PATHSEP);
 
 	/* Set defaults for Printer */
 	ConfigureParams.Printer.bEnablePrinting = FALSE;
 	ConfigureParams.Printer.bPrintToFile = TRUE;
-	sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%chatari.prn", szWorkingDir,PATHSEP);
+	sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%chatari.prn",
+	        psHomeDir, PATHSEP);
 
 	/* Set defaults for RS232 */
 	ConfigureParams.RS232.bEnableRS232 = FALSE;
@@ -383,30 +408,40 @@ void Configuration_SetDefault(void)
 
 	/* Set defaults for Screen */
 	ConfigureParams.Screen.bFullScreen = FALSE;
-	ConfigureParams.Screen.FrameSkips = 1;
+	ConfigureParams.Screen.FrameSkips = 0;
 	ConfigureParams.Screen.bAllowOverscan = TRUE;
-	ConfigureParams.Screen.bForce8Bpp = FALSE;
+	ConfigureParams.Screen.nSpec512Threshold = 16;
+	ConfigureParams.Screen.nForceBpp = 0;
 	ConfigureParams.Screen.bZoomLowRes = FALSE;
 	ConfigureParams.Screen.MonitorType = MONITOR_TYPE_RGB;
 	ConfigureParams.Screen.bUseExtVdiResolutions = FALSE;
-	ConfigureParams.Screen.nVdiResolution = GEMRES_640x480;
-	ConfigureParams.Screen.nVdiColors = GEMCOLOUR_16;
+	ConfigureParams.Screen.nVdiWidth = 640;
+	ConfigureParams.Screen.nVdiHeight = 480;
+	ConfigureParams.Screen.nVdiColors = GEMCOLOR_16;
 	ConfigureParams.Screen.bCaptureChange = FALSE;
 	ConfigureParams.Screen.nFramesPerSecond = 25;
+	ConfigureParams.Screen.nWindowBorderPixelsLeft = 48;
+	ConfigureParams.Screen.nWindowBorderPixelsRight = 48;
+	ConfigureParams.Screen.nWindowBorderPixelsBottom = OVERSCAN_BOTTOM;
+	ConfigureParams.Screen.nFullScreenBorderPixelsLeft = 32;
+	ConfigureParams.Screen.nFullScreenBorderPixelsRight = 32;
+	ConfigureParams.Screen.nFullScreenBorderPixelsBottom = OVERSCAN_BOTTOM;
 
 	/* Set defaults for Sound */
 	ConfigureParams.Sound.bEnableSound = TRUE;
-	ConfigureParams.Sound.nPlaybackQuality = PLAYBACK_MEDIUM;
-	sprintf(ConfigureParams.Sound.szYMCaptureFileName, "%s%chatari.wav", szWorkingDir,PATHSEP);
+	ConfigureParams.Sound.nPlaybackQuality = PLAYBACK_HIGH;
+	sprintf(ConfigureParams.Sound.szYMCaptureFileName, "%s%chatari.wav",
+	        psWorkingDir, PATHSEP);
 
 	/* Set defaults for Rom */
-	sprintf(ConfigureParams.Rom.szTosImageFileName, "%s%ctos.img", DATADIR,PATHSEP);
+	sprintf(ConfigureParams.Rom.szTosImageFileName, "%s%ctos.img",
+	        Paths_GetDataDir(), PATHSEP);
 	strcpy(ConfigureParams.Rom.szCartridgeImageFileName, "");
 
 	/* Set defaults for System */
 	ConfigureParams.System.nCpuLevel = 0;
 	ConfigureParams.System.nCpuFreq = 8;
-	ConfigureParams.System.bCompatibleCpu = FALSE;
+	ConfigureParams.System.bCompatibleCpu = TRUE;
 	/*ConfigureParams.System.bAddressSpace24 = TRUE;*/
 	ConfigureParams.System.nMachineType = MACHINE_ST;
 	ConfigureParams.System.bBlitter = FALSE;
@@ -417,15 +452,14 @@ void Configuration_SetDefault(void)
 	ConfigureParams.System.bSlowFDC = FALSE;
 
 	/* Initialize the configuration file name */
-	homeDir = getenv("HOME");
-	if (homeDir != NULL && homeDir[0] != 0 && strlen(homeDir) < sizeof(sConfigFileName)-13)
-		sprintf(sConfigFileName, "%s%c.hatari.cfg", homeDir,PATHSEP);
+	if (strlen(psHomeDir) < sizeof(sConfigFileName)-13)
+		sprintf(sConfigFileName, "%s%chatari.cfg", psHomeDir, PATHSEP);
 	else
 		strcpy(sConfigFileName, "hatari.cfg");
 
 #if defined(__AMIGAOS4__)
 	/* Fix default path names on Amiga OS */
-	sprintf(ConfigureParams.Rom.szTosImageFileName, "%stos.img", DATADIR);
+	sprintf(ConfigureParams.Rom.szTosImageFileName, "%stos.img", Paths_GetDataDir());
 #endif
 }
 
@@ -442,15 +476,16 @@ void Configuration_Apply(BOOL bReset)
 		/* Set resolution change */
 		bUseVDIRes = ConfigureParams.Screen.bUseExtVdiResolutions;
 		bUseHighRes = ((!bUseVDIRes) && ConfigureParams.Screen.MonitorType == MONITOR_TYPE_MONO)
-			|| (bUseVDIRes && ConfigureParams.Screen.nVdiColors == GEMCOLOUR_2);
+			|| (bUseVDIRes && ConfigureParams.Screen.nVdiColors == GEMCOLOR_2);
 		if (bUseHighRes)
 		{
 			STRes = ST_HIGH_RES;
 		}
 		if (bUseVDIRes)
 		{
-			VDI_SetResolution(ConfigureParams.Screen.nVdiResolution,
-			                  ConfigureParams.Screen.nVdiColors);
+			VDI_SetResolution(ConfigureParams.Screen.nVdiColors,
+			                  ConfigureParams.Screen.nVdiWidth,
+			                  ConfigureParams.Screen.nVdiHeight);
 		}
 	}
 
@@ -475,8 +510,7 @@ void Configuration_Apply(BOOL bReset)
 		nCpuFreqShift = 1;
 	}
 	/* Change UAE cpu_level and cpu_compatible accordingly */
-	check_prefs_changed_cpu(ConfigureParams.System.nCpuLevel,
-				ConfigureParams.System.bCompatibleCpu);
+	M68000_CheckCpuLevel();
 
 	/* Clean file and directory names */
 	File_MakeAbsoluteName(ConfigureParams.Rom.szTosImageFileName);
@@ -487,7 +521,8 @@ void Configuration_Apply(BOOL bReset)
 	File_MakeAbsoluteName(ConfigureParams.HardDisk.szHardDiskDirectories[0]);
 	File_MakeAbsoluteName(ConfigureParams.Memory.szMemoryCaptureFileName);
 	File_MakeAbsoluteName(ConfigureParams.Sound.szYMCaptureFileName);
-	File_MakeAbsoluteName(ConfigureParams.Keyboard.szMappingFileName);
+	if (strlen(ConfigureParams.Keyboard.szMappingFileName) > 0)
+		File_MakeAbsoluteName(ConfigureParams.Keyboard.szMappingFileName);
 	File_MakeAbsoluteName(ConfigureParams.RS232.szOutFileName);
 	File_MakeAbsoluteName(ConfigureParams.RS232.szInFileName);
 	
@@ -626,7 +661,8 @@ void Configuration_MemorySnapShot_Capture(BOOL bSave)
 
 	MemorySnapShot_Store(&ConfigureParams.Screen.MonitorType, sizeof(ConfigureParams.Screen.MonitorType));
 	MemorySnapShot_Store(&ConfigureParams.Screen.bUseExtVdiResolutions, sizeof(ConfigureParams.Screen.bUseExtVdiResolutions));
-	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiResolution, sizeof(ConfigureParams.Screen.nVdiResolution));
+	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiWidth, sizeof(ConfigureParams.Screen.nVdiWidth));
+	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiHeight, sizeof(ConfigureParams.Screen.nVdiHeight));
 	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiColors, sizeof(ConfigureParams.Screen.nVdiColors));
 
 	MemorySnapShot_Store(&ConfigureParams.System.nCpuLevel, sizeof(ConfigureParams.System.nCpuLevel));
