@@ -9,7 +9,7 @@
   open our dialog we make a backup of this structure. When the user finally
   clicks on 'OK', we can compare and makes the necessary changes.
 */
-const char Dialog_rcsid[] = "Hatari $Id: dialog.c,v 1.60 2007/01/30 20:33:49 eerot Exp $";
+const char Dialog_rcsid[] = "Hatari $Id: dialog.c,v 1.66 2008/03/01 22:37:43 eerot Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -21,6 +21,7 @@ const char Dialog_rcsid[] = "Hatari $Id: dialog.c,v 1.60 2007/01/30 20:33:49 eer
 #include "ioMem.h"
 #include "joy.h"
 #include "keymap.h"
+#include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "printer.h"
@@ -33,7 +34,7 @@ const char Dialog_rcsid[] = "Hatari $Id: dialog.c,v 1.60 2007/01/30 20:33:49 eer
 #include "vdi.h"
 #include "video.h"
 #include "sdlgui.h"
-#include "uae-cpu/hatari-glue.h"
+#include "hatari-glue.h"
 #if ENABLE_DSP_EMU
 # include "falcon/dsp.h"
 #endif
@@ -60,9 +61,10 @@ BOOL Dialog_DoNeedReset(void)
 	if (ConfigureParams.Screen.bUseExtVdiResolutions != DialogParams.Screen.bUseExtVdiResolutions)
 		return TRUE;
 
-	/* Did change GEM resolution or colour depth? */
+	/* Did change GEM resolution or color depth? */
 	if (DialogParams.Screen.bUseExtVdiResolutions &&
-	    (ConfigureParams.Screen.nVdiResolution != DialogParams.Screen.nVdiResolution
+	    (ConfigureParams.Screen.nVdiWidth != DialogParams.Screen.nVdiWidth
+	     || ConfigureParams.Screen.nVdiHeight != DialogParams.Screen.nVdiHeight
 	     || ConfigureParams.Screen.nVdiColors != DialogParams.Screen.nVdiColors))
 		return TRUE;
 
@@ -109,11 +111,11 @@ void Dialog_CopyDialogParamsToConfiguration(BOOL bForceReset)
 	/* Do need to change resolution? Need if change display/overscan settings */
 	/*(if switch between Colour/Mono cause reset later) */
 	if (!NeedReset &&
-	    (DialogParams.Screen.bForce8Bpp != ConfigureParams.Screen.bForce8Bpp
+	    (DialogParams.Screen.nForceBpp != ConfigureParams.Screen.nForceBpp
 	     || DialogParams.Screen.bZoomLowRes != ConfigureParams.Screen.bZoomLowRes
 	     || DialogParams.Screen.bAllowOverscan != ConfigureParams.Screen.bAllowOverscan))
 	{
-		ConfigureParams.Screen.bForce8Bpp = DialogParams.Screen.bForce8Bpp;
+		ConfigureParams.Screen.nForceBpp = DialogParams.Screen.nForceBpp;
 		ConfigureParams.Screen.bZoomLowRes = DialogParams.Screen.bZoomLowRes;
 		ConfigureParams.Screen.bAllowOverscan = DialogParams.Screen.bAllowOverscan;
 
@@ -247,7 +249,7 @@ void Dialog_CopyDialogParamsToConfiguration(BOOL bForceReset)
 /*-----------------------------------------------------------------------*/
 /**
  * Open Property sheet Options dialog.
- * Return TRUE if user choses OK, or FALSE if cancel!
+ * Return TRUE if user chooses OK, or FALSE if cancel!
  */
 BOOL Dialog_DoProperty(void)
 {
@@ -264,14 +266,24 @@ BOOL Dialog_DoProperty(void)
 
 	bOKDialog = Dialog_MainDlg(&bForceReset);
 
-	/* Copy details to configuration, and ask user if wishes to reset */
-	if (bOKDialog)
+	/* Check if reset is required and ask user if he really wants to continue then */
+	if (bOKDialog && !bForceReset && Dialog_DoNeedReset()
+	    && ConfigureParams.Log.nAlertDlgLogLevel >= LOG_INFO) {
+		bOKDialog = DlgAlert_Query("The emulated system must be "
+		                           "reset to apply these changes. "
+		                           "Apply changes now and reset "
+		                           "the emulator?");
+	}
+
+	/* Copy details to configuration */
+	if (bOKDialog) {
 		Dialog_CopyDialogParamsToConfiguration(bForceReset);
+	}
 
 	Main_UnPauseEmulation();
 
 	if (bQuitProgram)
-		set_special(SPCFLAG_BRK);           /* Assure that CPU core shuts down */
+		Main_RequestQuit();
 
 	return bOKDialog;
 }
@@ -285,11 +297,11 @@ void Dialog_LoadParams(void)
 {
 	CNF_PARAMS tmpParams;
 	/* Configuration_Load uses the variables from ConfigureParams.
-    * That's why we have to temporarily back it up here */
-    tmpParams = ConfigureParams;
-    Configuration_Load(NULL);
-    DialogParams = ConfigureParams;
-    ConfigureParams = tmpParams;
+	 * That's why we have to temporarily back it up here */
+	tmpParams = ConfigureParams;
+	Configuration_Load(NULL);
+	DialogParams = ConfigureParams;
+	ConfigureParams = tmpParams;
 }
 
 
@@ -301,9 +313,9 @@ void Dialog_SaveParams(void)
 {
 	CNF_PARAMS tmpParams;
 	/* Configuration_Save uses the variables from ConfigureParams.
-    * That's why we have to temporarily back it up here */
-    tmpParams = ConfigureParams;
-    ConfigureParams = DialogParams;
-    Configuration_Save();
-    ConfigureParams = tmpParams;
+	 * That's why we have to temporarily back it up here */
+	tmpParams = ConfigureParams;
+	ConfigureParams = DialogParams;
+	Configuration_Save();
+	ConfigureParams = tmpParams;
 }
