@@ -32,16 +32,18 @@
     $FF8922 (byte) : Microwire Data Register
     $FF8924 (byte) : Microwire Mask Register
 */
-const char DmaSnd_rcsid[] = "Hatari $Id: dmaSnd.c,v 1.8 2006/02/12 21:28:22 eerot Exp $";
+const char DmaSnd_rcsid[] = "Hatari $Id: dmaSnd.c,v 1.15 2008/01/28 22:20:09 thothy Exp $";
 
 #include "main.h"
 #include "audio.h"
+#include "configuration.h"
 #include "dmaSnd.h"
 #include "int.h"
 #include "ioMem.h"
 #include "memorySnapShot.h"
 #include "mfp.h"
 #include "sound.h"
+#include "stMemory.h"
 
 
 Uint16 nDmaSoundControl;                /* Sound control register */
@@ -56,7 +58,7 @@ static Uint32 nFrameEndAddr;            /* Sound frame end */
 static double FrameCounter;             /* Counter in current sound frame */
 static int nFrameLen;                   /* Length of the frame */
 
-static const int DmaSndSampleRates[4] =
+static const double DmaSndSampleRates[4] =
 {
 	6258,
 	12517,
@@ -65,10 +67,30 @@ static const int DmaSndSampleRates[4] =
 };
 
 
+static const double DmaSndFalcSampleRates[] =
+{
+	49170,
+	32780,
+	24585,
+	19668,
+	16390,
+	14049,
+	12292,
+	10927,
+	 9834,
+	 8940,
+	 8195,
+	 7565,
+	 7024,
+	 6556,
+	 6146,
+};
+
+
 /*-----------------------------------------------------------------------*/
-/*
-  Reset DMA sound variables.
-*/
+/**
+ * Reset DMA sound variables.
+ */
 void DmaSnd_Reset(BOOL bCold)
 {
 	nDmaSoundControl = 0;
@@ -81,9 +103,9 @@ void DmaSnd_Reset(BOOL bCold)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Save/Restore snapshot of local variables ('MemorySnapShot_Store' handles type)
-*/
+/**
+ * Save/Restore snapshot of local variables ('MemorySnapShot_Store' handles type)
+ */
 void DmaSnd_MemorySnapShot_Capture(BOOL bSave)
 {
 	/* Save/Restore details */
@@ -99,12 +121,27 @@ void DmaSnd_MemorySnapShot_Capture(BOOL bSave)
 }
 
 
+static double DmaSnd_DetectSampleRate(void)
+{
+	int nFalcClk = IoMem[0xff8935] & 0x0f;
+
+	if (ConfigureParams.System.nMachineType == MACHINE_FALCON && nFalcClk != 0)
+	{
+		return DmaSndFalcSampleRates[nFalcClk-1];
+	}
+	else
+	{
+		return DmaSndSampleRates[nDmaSoundMode & 3];
+	}
+}
+
+
 /*-----------------------------------------------------------------------*/
-/*
-  This function is called when a new sound frame is started.
-  It copies the start and end address from the I/O registers, calculates
-  the sample length, etc.
-*/
+/**
+ * This function is called when a new sound frame is started.
+ * It copies the start and end address from the I/O registers, calculates
+ * the sample length, etc.
+ */
 static void DmaSnd_StartNewFrame(void)
 {
 	int nCyclesForFrame;
@@ -117,19 +154,19 @@ static void DmaSnd_StartNewFrame(void)
 
 	/* To get smooth sound, set an "interrupt" for the end of the frame that
 	 * updates the sound mix buffer. */
-	nCyclesForFrame = nFrameLen * (8013000.0 / (double)DmaSndSampleRates[nDmaSoundMode & 3]);
+	nCyclesForFrame = nFrameLen * (8013000.0 / DmaSnd_DetectSampleRate());
 	if (!(nDmaSoundMode & DMASNDMODE_MONO))  /* Is it stereo? */
 		nCyclesForFrame = nCyclesForFrame / 2;
-	Int_AddRelativeInterrupt(nCyclesForFrame, INTERRUPT_DMASOUND);
+	Int_AddRelativeInterrupt(nCyclesForFrame, INT_CPU_CYCLE, INTERRUPT_DMASOUND, 0);
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Check if end-of-frame has been reached and raise interrupts if needed.
-  Returns TRUE if DMA sound processing should be stopped now and FALSE
-  if it continues.
-*/
+/**
+ * Check if end-of-frame has been reached and raise interrupts if needed.
+ * Returns TRUE if DMA sound processing should be stopped now and FALSE
+ * if it continues.
+ */
 static inline int DmaSnd_CheckForEndOfFrame(int nFrameCounter)
 {
 	if (nFrameCounter >= nFrameLen)
@@ -155,9 +192,9 @@ static inline int DmaSnd_CheckForEndOfFrame(int nFrameCounter)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Mix DMA sound sample with the normal PSG sound samples.
-*/
+/**
+ * Mix DMA sound sample with the normal PSG sound samples.
+ */
 void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 {
 	double FreqRatio;
@@ -169,12 +206,26 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 		return;
 
 	pFrameStart = (Sint8 *)&STRam[nFrameStartAddr];
-	FreqRatio = (double)DmaSndSampleRates[nDmaSoundMode & 3]
-	            / (double)SoundPlayBackFrequencies[OutputAudioFreqIndex];
+	FreqRatio = DmaSnd_DetectSampleRate() / (double)SoundPlayBackFrequencies[OutputAudioFreqIndex];
 
-	if (nDmaSoundMode & DMASNDMODE_MONO)  /* Stereo or mono? */
+	if (ConfigureParams.System.nMachineType == MACHINE_FALCON
+	    && (nDmaSoundMode & DMASNDMODE_16BITSTEREO))
 	{
-		/* Mono */
+		/* Stereo 16-bit */
+		FreqRatio *= 4.0;
+		for (i = 0; i < nSamplesToGenerate; i++)
+		{
+			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			MixBuffer[nBufIdx] = ((int)MixBuffer[nBufIdx] + (int)pFrameStart[((int)FrameCounter)&~1]
+			                      + (int)pFrameStart[(((int)FrameCounter)&~1)+2]) / 3;
+			FrameCounter += FreqRatio;
+			if (DmaSnd_CheckForEndOfFrame(FrameCounter))
+				break;
+		}
+	}
+	else if (nDmaSoundMode & DMASNDMODE_MONO)  /* 8-bit stereo or mono? */
+	{
+		/* Mono 8-bit */
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
 			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
@@ -186,7 +237,7 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 	}
 	else
 	{
-		/* Stereo */
+		/* Stereo 8-bit */
 		FreqRatio *= 2.0;
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
@@ -202,10 +253,10 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  DMA sound end of frame "interrupt". Used for updating the sound after
-  a frame has been finished.
-*/
+/**
+ * DMA sound end of frame "interrupt". Used for updating the sound after
+ * a frame has been finished.
+ */
 void DmaSnd_InterruptHandler(void)
 {
 	/* Remove this interrupt from list and re-order */
@@ -220,9 +271,9 @@ void DmaSnd_InterruptHandler(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Create actual position for frame count registers.
-*/
+/**
+ * Create actual position for frame count registers.
+ */
 static Uint32 DmaSnd_GetFrameCount(void)
 {
 	Uint32 nActCount;
@@ -239,9 +290,9 @@ static Uint32 DmaSnd_GetFrameCount(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from sound control register (0xff8900).
-*/
+/**
+ * Read word from sound control register (0xff8900).
+ */
 void DmaSnd_SoundControl_ReadWord(void)
 {
 	IoMem_WriteWord(0xff8900, nDmaSoundControl);
@@ -249,9 +300,10 @@ void DmaSnd_SoundControl_ReadWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write word to sound control register (0xff8900).
-*/
+/**
+ * Write word to sound control register (0xff8900).
+ */
+/* FIXME: add Falcon specific handler here... */
 void DmaSnd_SoundControl_WriteWord(void)
 {
 	Uint16 nNewSndCtrl;
@@ -273,9 +325,9 @@ void DmaSnd_SoundControl_WriteWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from sound frame count high register (0xff8909).
-*/
+/**
+ * Read word from sound frame count high register (0xff8909).
+ */
 void DmaSnd_FrameCountHigh_ReadByte(void)
 {
 	IoMem_WriteByte(0xff8909, DmaSnd_GetFrameCount() >> 16);
@@ -283,9 +335,9 @@ void DmaSnd_FrameCountHigh_ReadByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from sound frame count medium register (0xff890b).
-*/
+/**
+ * Read word from sound frame count medium register (0xff890b).
+ */
 void DmaSnd_FrameCountMed_ReadByte(void)
 {
 	IoMem_WriteByte(0xff890b, DmaSnd_GetFrameCount() >> 8);
@@ -293,9 +345,9 @@ void DmaSnd_FrameCountMed_ReadByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from sound frame count low register (0xff890d).
-*/
+/**
+ * Read word from sound frame count low register (0xff890d).
+ */
 void DmaSnd_FrameCountLow_ReadByte(void)
 {
 	IoMem_WriteByte(0xff890d, DmaSnd_GetFrameCount());
@@ -303,9 +355,9 @@ void DmaSnd_FrameCountLow_ReadByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from sound mode register (0xff8920).
-*/
+/**
+ * Read word from sound mode register (0xff8920).
+ */
 void DmaSnd_SoundMode_ReadWord(void)
 {
 	IoMem_WriteWord(0xff8920, nDmaSoundMode);
@@ -313,20 +365,38 @@ void DmaSnd_SoundMode_ReadWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write word to sound mode register (0xff8920).
-*/
+/**
+ * Write word to sound mode register (0xff8920).
+ * Handling framework for Falcon specific bits by Matthias Arndt <marndt@asmsoftware.de>
+ */
 void DmaSnd_SoundMode_WriteWord(void)
 {
-	nDmaSoundMode = IoMem_ReadWord(0xff8920);
+	/* Handle Falcon specialities: STE and TT only have bits 7,1 and 0 in this register */
+
+	/* Falcon has meaning in almost all bits of SND_SMC */
+	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+	{
+		
+		nDmaSoundMode = IoMem_ReadWord(0xff8920);
+		/* FIXME: add code here to evaluate Falcon specific settings */
+		
+		
+	} else {
+		/* STE or TT - hopefully STFM emulation never gets here :)
+		 * we maskout the Falcon only bits so we only hit bits that exist on a real STE
+		 */
+		nDmaSoundMode = (IoMem_ReadWord(0xff8920)&0x008F);
+		/* we also write the masked value back into the emulated hw registers so we have a correct value there */
+		IoMem_WriteWord(0xff8920,nDmaSoundMode);
+	}
 	//fprintf(stderr,"New sound mode = $%x\n", nDmaSoundMode);
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from microwire data register (0xff8922).
-*/
+/**
+ * Read word from microwire data register (0xff8922).
+ */
 void DmaSnd_MicrowireData_ReadWord(void)
 {
 	/* The Microwire shifts the data register bit by bit until it is zero
@@ -345,9 +415,9 @@ void DmaSnd_MicrowireData_ReadWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write word to microwire data register (0xff8922).
-*/
+/**
+ * Write word to microwire data register (0xff8922).
+ */
 void DmaSnd_MicrowireData_WriteWord(void)
 {
 	nMicrowireData = IoMem_ReadWord(0xff8922);
@@ -356,9 +426,9 @@ void DmaSnd_MicrowireData_WriteWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read word from microwire mask register (0xff8924).
-*/
+/**
+ * Read word from microwire mask register (0xff8924).
+ */
 void DmaSnd_MicrowireMask_ReadWord(void)
 {
 	/* Same as with data register, but mask register is rotated, not  shifted. */
@@ -376,9 +446,9 @@ void DmaSnd_MicrowireMask_ReadWord(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write word to microwire mask register (0xff8924).
-*/
+/**
+ * Write word to microwire mask register (0xff8924).
+ */
 void DmaSnd_MicrowireMask_WriteWord(void)
 {
 	nMicrowireMask = IoMem_ReadWord(0xff8924);
