@@ -12,7 +12,10 @@
   checked each HBL to perform the transfer of data from our disk image into
   the ST RAM area by simulating the DMA.
 */
-const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.31 2007/01/16 18:42:59 thothy Exp $";
+
+/* 2007/11/06   [NP]    Add calls to HATARI_TRACE and set FDC_DELAY_HBL=180		*/
+
+const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.35 2008/03/13 23:17:27 thothy Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -28,6 +31,7 @@ const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.31 2007/01/16 18:42:59 thothy Ex
 #include "misc.h"
 #include "psg.h"
 #include "stMemory.h"
+#include "trace.h"
 
 
 /*
@@ -157,8 +161,6 @@ static short int nReadWriteSectors;
 
 static Uint8 DMASectorWorkSpace[NUMBYTESPERSECTOR];             /* Workspace used to copy to/from for floppy DMA */
 
-static int nFdcDelayHbls;                                       /* Used to slow down FDC emulation */
-
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -421,7 +423,11 @@ void FDC_GpipRead(void)
 	if ((MFP_GPIP & 0x20) == nLastGpipBit)
 	{
 		if (!ConfigureParams.System.bSlowFDC)
-			nFdcDelayHbls = 0;
+		{
+			/* Restart FDC update interrupt to occur right after a few cycles */
+			Int_RemovePendingInterrupt(INTERRUPT_FDC);
+			Int_AddRelativeInterrupt(4, INT_CPU_CYCLE, INTERRUPT_FDC, 0);
+		}
 	}
 	else
 	{
@@ -432,17 +438,15 @@ void FDC_GpipRead(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Update floppy drive on each HBL (approx' 512 cycles)
+ * Update floppy drive after a while.
+ * Some games/demos (e.g. Fantasia by Dune, Alien World, ...) don't work
+ * if the FDC is too fast. So we use the update "interrupt" for delayed
+ * execution of the commands.
+ * FIXME: We urgently need better timings here!
  */
-void FDC_UpdateHBL(void)
+void FDC_InterruptHandler_Update(void)
 {
-	/* Seems like some games/demos (e.g. Fantasia by Dune, Alien World, ...) don't
-	 * work if the FDC is too fast... so here's a quick-n-dirty hack to get them
-	 * working... Should be replaced with proper FDC timings one day! */
-	if (nFdcDelayHbls-- > 0)
-		return;
-	else
-		nFdcDelayHbls = 180;
+	Int_AcknowledgeInterrupt();
 
 	/* Do we have a DMA ready to copy? */
 	if (bDMAWaiting)
@@ -491,6 +495,11 @@ void FDC_UpdateHBL(void)
 		/* Set disk controller status (RD 0xff8604) */
 		FDC_SetDiskControllerStatus();
 	}
+
+	if (FDCEmulationCommand != FDCEMU_CMD_NULL || bMotorOn)
+	{
+		Int_AddAbsoluteInterrupt(FDC_DELAY_CYCLES,  INT_CPU_CYCLE, INTERRUPT_FDC);
+	}
 }
 
 
@@ -1074,6 +1083,8 @@ static void FDC_ExecuteCommand(void)
 		FDC_ExecuteTypeIIICommands();
 	else                                          /* Type IV - Force Interrupt */
 		FDC_ExecuteTypeIVCommands();
+
+	Int_AddAbsoluteInterrupt(FDC_DELAY_CYCLES,  INT_CPU_CYCLE, INTERRUPT_FDC);
 }
 
 
@@ -1083,6 +1094,9 @@ static void FDC_ExecuteCommand(void)
  */
 static void FDC_WriteSectorCountRegister(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector count=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	FDCSectorCountRegister = DiskControllerWord_ff8604wr;
 }
 
@@ -1093,6 +1107,9 @@ static void FDC_WriteSectorCountRegister(void)
  */
 static void FDC_WriteCommandRegister(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 command=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	FDCCommandRegister = DiskControllerWord_ff8604wr;
 	/* And execute */
 	FDC_ExecuteCommand();
@@ -1105,6 +1122,9 @@ static void FDC_WriteCommandRegister(void)
  */
 static void FDC_WriteTrackRegister(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 track=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	FDCTrackRegister = DiskControllerWord_ff8604wr;    /* 0...79 */
 }
 
@@ -1115,6 +1135,9 @@ static void FDC_WriteTrackRegister(void)
  */
 static void FDC_WriteSectorRegister(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	FDCSectorRegister = DiskControllerWord_ff8604wr;  /* 1,2,3..... */
 }
 
@@ -1125,6 +1148,9 @@ static void FDC_WriteSectorRegister(void)
  */
 static void FDC_WriteDataRegister(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	FDCDataRegister = DiskControllerWord_ff8604wr;
 }
 
@@ -1146,6 +1172,9 @@ void FDC_DiskController_WriteWord(void)
 
 	DiskControllerWord_ff8604wr = IoMem_ReadWord(0xff8604);
 
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x video_cyc=%d\n" ,
+		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	HDC_WriteCommandPacket();                 /*  Handle HDC functions */
 
 	/* filter hdc commands */
@@ -1244,6 +1273,9 @@ void FDC_DiskControllerStatus_ReadWord(void)
 	}
 
 	IoMem_WriteWord(0xff8604, DiskControllerByte);
+
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read 8604 ctrl status=0x%x video_cyc=%d\n" ,
+		DiskControllerByte , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
 }
 
 
@@ -1268,6 +1300,8 @@ Uint32 FDC_ReadDMAAddress(void)
  */
 void FDC_WriteDMAAddress(Uint32 Address)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 0x%x to dma address video_cyc=%d\n" , Address , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	/* Store as 24-bit address */
 	STMemory_WriteByte(0xff8609, Address>>16);
 	STMemory_WriteByte(0xff860b, Address>>8);
@@ -1282,6 +1316,9 @@ void FDC_WriteDMAAddress(Uint32 Address)
  */
 BOOL FDC_ReadSectorFromFloppy(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read sector dev=%d sect=%d track=%d side=%d video_cyc=%d\n" ,
+		nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	/* Copy in 1 sector to our workspace */
 	if (Floppy_ReadSectors(nReadWriteDev, DMASectorWorkSpace, nReadWriteSector, nReadWriteTrack, nReadWriteSide, 1, NULL))
 	{
@@ -1296,6 +1333,7 @@ BOOL FDC_ReadSectorFromFloppy(void)
 	}
 
 	/* Failed */
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read sector failed\n" );
 	return FALSE;
 }
 
@@ -1312,6 +1350,9 @@ BOOL FDC_WriteSectorFromFloppy(void)
 	/* Get DMA address */
 	Address = FDC_ReadDMAAddress();
 
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write sector addr=%x dev=%d sect=%d track=%d side=%d video_cyc=%d\n" ,
+		Address, nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	/* Write out 1 sector from our workspace */
 	if (Floppy_WriteSectors(nReadWriteDev, &STRam[Address], nReadWriteSector, nReadWriteTrack, nReadWriteSide, 1, NULL))
 	{
@@ -1326,6 +1367,7 @@ BOOL FDC_WriteSectorFromFloppy(void)
 	}
 
 	/* Failed */
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write sector failed\n" );
 	return FALSE;
 }
 
@@ -1370,7 +1412,58 @@ void FDC_DmaModeControl_WriteWord(void)
 	DMAModeControl_ff8606wr_prev = DMAModeControl_ff8606wr;  /* Store previous to check for _read/_write toggle (DMA reset) */
 	DMAModeControl_ff8606wr = IoMem_ReadWord(0xff8606);      /* Store to DMA Mode control */
 
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8606 ctrl=0x%x video_cyc=%d\n" , DMAModeControl_ff8606wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+
 	/* When write to 0xff8606, check bit '8' toggle. This causes DMA status reset */
 	if ((DMAModeControl_ff8606wr_prev ^ DMAModeControl_ff8606wr) & 0x0100)
 		FDC_ResetDMAStatus();
 }
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Write to floppy mode/control (?) register (0xff860F).
+ * Used on Falcon only!
+ * FIXME: I've found hardly any documentation about this register, only
+ * the following description of the bits:
+ *
+ *   __________54__10  Floppy Controll-Register
+ *             ||  ||
+ *             ||  |+- Prescaler 1
+ *             ||  +-- Media detect 1
+ *             |+----- Prescaler 2
+ *             +------ Media detect 2
+ *
+ * For DD - disks:  0x00
+ * For HD - disks:  0x03
+ * for ED - disks:  0x30 (not supported by TOS)
+ */
+void FDC_FloppyMode_WriteByte(void)
+{
+	// printf("Write to floppy mode reg.: 0x%02x\n", IoMem_ReadByte(0xff860f));
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Read from floppy mode/control (?) register (0xff860F).
+ * Used on Falcon only!
+ * FIXME: I've found hardly any documentation about this register, only
+ * the following description of the bits:
+ *
+ *   ________76543210  Floppy Controll-Register
+ *           ||||||||
+ *           |||||||+- Prescaler 1
+ *           ||||||+-- Mode select 1
+ *           |||||+--- Media detect 1
+ *           ||||+---- accessed during DMA transfers (?)
+ *           |||+----- Prescaler 2
+ *           ||+------ Mode select 2
+ *           |+------- Media detect 2
+ *           +-------- Disk changed
+ */
+void FDC_FloppyMode_ReadByte(void)
+{
+	IoMem_WriteByte(0xff860f, 0x80);  // FIXME: Is this ok?
+	// printf("Read from floppy mode reg.: 0x%02x\n", IoMem_ReadByte(0xff860f));
+}
