@@ -6,10 +6,11 @@
 
   Common file access functions.
 */
-const char File_rcsid[] = "Hatari $Id: file.c,v 1.41 2007/02/27 20:53:52 eerot Exp $";
+const char File_rcsid[] = "Hatari $Id: file.c,v 1.51 2008/03/18 00:56:32 thothy Exp $";
+
+#include <config.h>
 
 #include <string.h>
-#include <strings.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -18,6 +19,10 @@ const char File_rcsid[] = "Hatari $Id: file.c,v 1.41 2007/02/27 20:53:52 eerot E
 #include <errno.h>
 #include <zlib.h>
 
+#if HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include "main.h"
 #include "dialog.h"
 #include "file.h"
@@ -35,16 +40,9 @@ void File_CleanFileName(char *pszFileName)
 
 	len = strlen(pszFileName);
 
-	/* Security length check: */
-	if (len > FILENAME_MAX)
-	{
-		pszFileName[FILENAME_MAX-1] = 0;
-		len = FILENAME_MAX;
-	}
-
 	/* Remove end slash from filename! But / remains! Doh! */
 	if (len > 2 && pszFileName[len-1] == PATHSEP)
-		pszFileName[len-1] = 0;
+		pszFileName[len-1] = '\0';
 }
 
 
@@ -61,10 +59,10 @@ void File_AddSlashToEndFileName(char *pszFileName)
 	/* Check dir/filenames */
 	if (len != 0)
 	{
-		if (pszFileName[strlen(pszFileName)-1] != PATHSEP)
+		if (pszFileName[len-1] != PATHSEP)
 		{
 			pszFileName[len] = PATHSEP; /* Must use end slash */
-			pszFileName[len+1] = 0;
+			pszFileName[len+1] = '\0';
 		}
 	}
 }
@@ -93,17 +91,17 @@ BOOL File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtensi
  * 
  * Return TRUE if filename is '/', else give FALSE
  */
-BOOL File_IsRootFileName(char *pszFileName)
+static BOOL File_IsRootFileName(const char *pszFileName)
 {
-	if (pszFileName[0]=='\0')     /* If NULL string return! */
+	if (pszFileName[0] == '\0')     /* If NULL string return! */
 		return FALSE;
 
-	if (pszFileName[0]==PATHSEP)
+	if (pszFileName[0] == PATHSEP)
 		return TRUE;
 
 #ifdef WIN32
 
-	if (pszFileName[1]==':')
+	if (pszFileName[1] == ':')
 		return TRUE;
 #endif
 
@@ -117,7 +115,7 @@ BOOL File_IsRootFileName(char *pszFileName)
  */
 const char *File_RemoveFileNameDrive(const char *pszFileName)
 {
-	if ( (pszFileName[0]!='\0') && (pszFileName[1]==':') )
+	if ( (pszFileName[0] != '\0') && (pszFileName[1] == ':') )
 		return &pszFileName[2];
 	else
 		return pszFileName;
@@ -134,38 +132,42 @@ BOOL File_DoesFileNameEndWithSlash(char *pszFileName)
 {
 	if (pszFileName[0] == '\0')    /* If NULL string return! */
 		return FALSE;
-
+	
 	/* Does string end in a '/'? */
 	if (pszFileName[strlen(pszFileName)-1] == PATHSEP)
 		return TRUE;
-
+	
 	return FALSE;
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Read file from disk into memory, allocate memory for it if need to (pass
- * Address as NULL).
+ * Read file from disk into allocated buffer and return the buffer
+ * or NULL for error.  If pFileSize is non-NULL, read file size
+ * is set to that.
  */
-void *File_Read(char *pszFileName, void *pAddress, long *pFileSize, const char * const ppszExts[])
+Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const ppszExts[])
 {
+	char *filepath = NULL;
 	void *pFile = NULL;
 	long FileSize = 0;
 
 	/* Does the file exist? If not, see if can scan for other extensions and try these */
 	if (!File_Exists(pszFileName) && ppszExts)
 	{
-		/* Try other extensions, if suceeds correct filename is now in 'pszFileName' */
-		File_FindPossibleExtFileName(pszFileName, ppszExts);
+		/* Try other extensions, if succeeds, returns correct one */
+		filepath = File_FindPossibleExtFileName(pszFileName, ppszExts);
 	}
+	if (!filepath)
+		filepath = strdup(pszFileName);
 
 	/* Is it a gzipped file? */
-	if (File_DoesFileExtensionMatch(pszFileName, ".gz"))
+	if (File_DoesFileExtensionMatch(filepath, ".gz"))
 	{
 		gzFile hGzFile;
 		/* Open and read gzipped file */
-		hGzFile = gzopen(pszFileName, "rb");
+		hGzFile = gzopen(filepath, "rb");
 		if (hGzFile != NULL)
 		{
 			/* Find size of file: */
@@ -177,52 +179,39 @@ void *File_Read(char *pszFileName, void *pAddress, long *pFileSize, const char *
 			while (!gzeof(hGzFile));
 			FileSize = gztell(hGzFile);
 			gzrewind(hGzFile);
-			/* Find pointer to where to load, allocate memory if pass NULL */
-			if (pAddress)
-				pFile = pAddress;
-			else
-				pFile = malloc(FileSize);
 			/* Read in... */
+			pFile = malloc(FileSize);
 			if (pFile)
 				FileSize = gzread(hGzFile, pFile, FileSize);
 
 			gzclose(hGzFile);
 		}
 	}
-	else if (File_DoesFileExtensionMatch(pszFileName, ".zip"))
+	else if (File_DoesFileExtensionMatch(filepath, ".zip"))
 	{
 		/* It is a .ZIP file! -> Try to load the first file in the archive */
-		pFile = ZIP_ReadFirstFile(pszFileName, &FileSize, ppszExts);
-		if (pFile && pAddress)
-		{
-			memcpy(pAddress, pFile, FileSize);
-			free(pFile);
-			pFile = pAddress;
-		}
+		pFile = ZIP_ReadFirstFile(filepath, &FileSize, ppszExts);
 	}
 	else          /* It is a normal file */
 	{
 		FILE *hDiskFile;
 		/* Open and read normal file */
-		hDiskFile = fopen(pszFileName, "rb");
+		hDiskFile = fopen(filepath, "rb");
 		if (hDiskFile != NULL)
 		{
 			/* Find size of file: */
 			fseek(hDiskFile, 0, SEEK_END);
 			FileSize = ftell(hDiskFile);
 			fseek(hDiskFile, 0, SEEK_SET);
-			/* Find pointer to where to load, allocate memory if pass NULL */
-			if (pAddress)
-				pFile = pAddress;
-			else
-				pFile = malloc(FileSize);
 			/* Read in... */
+			pFile = malloc(FileSize);
 			if (pFile)
 				FileSize = fread(pFile, 1, FileSize, hDiskFile);
 
 			fclose(hDiskFile);
 		}
 	}
+	free(filepath);
 
 	/* Store size of file we read in (or 0 if failed) */
 	if (pFileSize)
@@ -236,7 +225,7 @@ void *File_Read(char *pszFileName, void *pAddress, long *pFileSize, const char *
 /**
  * Save file to disk, return FALSE if errors
  */
-BOOL File_Save(char *pszFileName, const void *pAddress, size_t Size, BOOL bQueryOverwrite)
+BOOL File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, BOOL bQueryOverwrite)
 {
 	BOOL bRet = FALSE;
 
@@ -314,7 +303,8 @@ BOOL File_Exists(const char *filename)
 {
 	struct stat buf;
 	if (stat(filename, &buf) == 0 &&
-	    (buf.st_mode & (S_IRUSR|S_IWUSR)) && !(buf.st_mode & S_IFDIR)) {
+	    (buf.st_mode & (S_IRUSR|S_IWUSR)) && !(buf.st_mode & S_IFDIR))
+	{
 		/* file points to user readable regular file */
 		return TRUE;
 	}
@@ -322,72 +312,91 @@ BOOL File_Exists(const char *filename)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Return TRUE if directory exists. 
+ */
+BOOL File_DirectoryExists(const char *psDirName)
+{
+	struct stat buf;
+	if (stat(psDirName, &buf) == 0 && (buf.st_mode & S_IFDIR))
+	{
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Find if file exists, and if so ask user if OK to overwrite
  */
 BOOL File_QueryOverwrite(const char *pszFileName)
 {
-	char szString[FILENAME_MAX + 26];
+	const char *fmt;
+	char *szString;
+	BOOL ret = TRUE;
 
 	/* Try and find if file exists */
 	if (File_Exists(pszFileName))
 	{
+		fmt = "File '%s' exists, overwrite?";
 		/* File does exist, are we OK to overwrite? */
-		snprintf(szString, sizeof(szString), "File '%s' exists, overwrite?", pszFileName);
+		szString = malloc(strlen(pszFileName) + strlen(fmt) + 1);
+		sprintf(szString, fmt, pszFileName);
 		fprintf(stderr, "%s\n", szString);
-		return DlgAlert_Query(szString);
+		ret = DlgAlert_Query(szString);
+		free(szString);
 	}
-
-	return TRUE;
+	return ret;
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Try filename with various extensions and check if file exists - if so return correct name
+ * Try filename with various extensions and check if file exists
+ * - if so, return allocated string which caller should free,
+ *   otherwise return NULL
  */
-BOOL File_FindPossibleExtFileName(char *pszFileName, const char * const ppszExts[])
+char * File_FindPossibleExtFileName(const char *pszFileName, const char * const ppszExts[])
 {
 	char *szSrcDir, *szSrcName, *szSrcExt;
-	char *szTempFileName;
-	int i = 0;
-	BOOL bFileExists = FALSE;
-
+	int i;
+	
 	/* Allocate temporary memory for strings: */
-	szTempFileName = malloc(4 * FILENAME_MAX);
-	if (!szTempFileName)
+	szSrcDir = malloc(3 * FILENAME_MAX);
+	if (!szSrcDir)
 	{
 		perror("File_FindPossibleExtFileName");
 		return FALSE;
 	}
-	szSrcDir = szTempFileName + FILENAME_MAX;
 	szSrcName = szSrcDir + FILENAME_MAX;
 	szSrcExt = szSrcName + FILENAME_MAX;
-
+	
 	/* Split filename into parts */
-	File_splitpath(pszFileName, szSrcDir, szSrcName, szSrcExt);
+	File_SplitPath(pszFileName, szSrcDir, szSrcName, szSrcExt);
 
 	/* Scan possible extensions */
-	while(ppszExts[i] && !bFileExists)
+	for (i = 0; ppszExts[i]; i++)
 	{
+		char *szTempFileName;
+
 		/* Re-build with new file extension */
-		File_makepath(szTempFileName, szSrcDir, szSrcName, ppszExts[i]);
-		/* Does this file exist? */
-		if (File_Exists(szTempFileName))
+		szTempFileName = File_MakePath(szSrcDir, szSrcName, ppszExts[i]);
+		if (szTempFileName)
 		{
-			/* Copy name for return */
-			strcpy(pszFileName, szTempFileName);
-			bFileExists = TRUE;
+			/* Does this file exist? */
+			if (File_Exists(szTempFileName))
+			{
+				free(szSrcDir);
+				/* return filename without extra strings */
+				return szTempFileName;
+			}
+			free(szTempFileName);
 		}
-
-		/* Next one */
-		i++;
 	}
-
-	free(szTempFileName);
-
-	return bFileExists;
+	free(szSrcDir);
+	return NULL;
 }
 
 
@@ -395,8 +404,9 @@ BOOL File_FindPossibleExtFileName(char *pszFileName, const char * const ppszExts
 /**
  * Split a complete filename into path, filename and extension.
  * If pExt is NULL, don't split the extension from the file name!
+ * It's safe for pSrcFileName and pDir to be the same string.
  */
-void File_splitpath(const char *pSrcFileName, char *pDir, char *pName, char *pExt)
+void File_SplitPath(const char *pSrcFileName, char *pDir, char *pName, char *pExt)
 {
 	char *ptr1, *ptr2;
 
@@ -404,14 +414,14 @@ void File_splitpath(const char *pSrcFileName, char *pDir, char *pName, char *pEx
 	ptr1 = strrchr(pSrcFileName, PATHSEP);
 	if (ptr1)
 	{
-		strcpy(pDir, pSrcFileName);
 		strcpy(pName, ptr1+1);
+		memmove(pDir, pSrcFileName, ptr1-pSrcFileName+1);
 		pDir[ptr1-pSrcFileName+1] = 0;
 	}
 	else
 	{
+ 		strcpy(pName, pSrcFileName);
 		sprintf(pDir, ".%c", PATHSEP);
-		strcpy(pName, pSrcFileName);
 	}
 
 	/* Build the raw filename: */
@@ -432,47 +442,61 @@ void File_splitpath(const char *pSrcFileName, char *pDir, char *pName, char *pEx
 
 /*-----------------------------------------------------------------------*/
 /**
- * Build a complete filename from path, filename and extension.
+ * Construct a complete filename from path, filename and extension.
+ * Return the constructed filename.
  * pExt can also be NULL.
  */
-void File_makepath(char *pDestFileName, const char *pDir, const char *pName, const char *pExt)
+char * File_MakePath(const char *pDir, const char *pName, const char *pExt)
 {
+	char *filepath;
 	int len;
 
-	strcpy(pDestFileName, pDir);
-	len = strlen(pDestFileName);
-
-	if (len == 0)
-		sprintf(pDestFileName, ".%c", PATHSEP);
-	else if (pDestFileName[len-1] != PATHSEP)
+	/* dir or "." + "/" + name + "." + ext + \0 */
+	len = strlen(pDir) + 2 + strlen(pName) + 1 + (pExt ? strlen(pExt) : 0) + 1;
+	filepath = malloc(len);
+	if (!filepath)
 	{
-		pDestFileName[len] = PATHSEP;
-		pDestFileName[len+1] = 0;
+		perror("File_MakePath");
+		return NULL;
 	}
-
-	strcat(pDestFileName, pName);
-
-	if (pExt != NULL)
+	if (!pDir[0])
 	{
-		if (strlen(pExt) > 0 && pExt[0] != '.')
-			strcat(pDestFileName, ".");
-		strcat(pDestFileName, pExt);
+		filepath[0] = '.';
+		filepath[1] = '\0';
+	} else {
+		strcpy(filepath, pDir);
 	}
+	len = strlen(filepath);
+	if (filepath[len-1] != PATHSEP)
+	{
+		filepath[len++] = PATHSEP;
+	}
+	strcpy(&filepath[len], pName);
+
+	if (pExt != NULL && pExt[0])
+	{
+		len += strlen(pName);
+		if (pExt[0] != '.')
+			strcat(&filepath[len++], ".");
+		strcat(&filepath[len], pExt);
+	}
+	return filepath;
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
  * Shrink a file name to a certain length and insert some dots if we cut
- * something away (usefull for showing file names in a dialog).
+ * something away (useful for showing file names in a dialog).
  */
-void File_ShrinkName(char *pDestFileName, char *pSrcFileName, int maxlen)
+void File_ShrinkName(char *pDestFileName, const char *pSrcFileName, int maxlen)
 {
 	int srclen = strlen(pSrcFileName);
 	if (srclen < maxlen)
 		strcpy(pDestFileName, pSrcFileName);  /* It fits! */
 	else
 	{
+		assert(maxlen > 6);
 		strncpy(pDestFileName, pSrcFileName, maxlen/2);
 		if (maxlen&1)  /* even or uneven? */
 			pDestFileName[maxlen/2-1] = 0;
@@ -604,11 +628,18 @@ void File_MakeAbsoluteName(char *pFileName)
 			/* Ignore "./" */
 			inpos += 2;
 		}
-		else if (pFileName[inpos] == '.' && pFileName[inpos+1] == '.' && pFileName[inpos+2] == PATHSEP)
+		else if (pFileName[inpos] == '.' && pFileName[inpos+1] == 0)
+		{
+			inpos += 1;        /* Ignore "." at the end of the path string */
+			if (outpos > 1)
+				pTempName[outpos - 1] = 0;   /* Remove the last slash, too */
+		}
+		else if (pFileName[inpos] == '.' && pFileName[inpos+1] == '.'
+		         && (pFileName[inpos+2] == PATHSEP || pFileName[inpos+2] == 0))
 		{
 			/* Handle "../" */
 			char *pSlashPos;
-			inpos += 3;
+			inpos += 2;
 			pTempName[outpos - 1] = 0;
 			pSlashPos = strrchr(pTempName, PATHSEP);
 			if (pSlashPos)
@@ -621,6 +652,20 @@ void File_MakeAbsoluteName(char *pFileName)
 				pTempName[0] = PATHSEP;
 				outpos = 1;
 			}
+			/* Were we already at the end of the string or is there more to come? */
+			if (pFileName[inpos] == PATHSEP)
+			{
+				/* There was a slash after the '..', so skip slash and
+				 * simply proceed with next part */
+				inpos += 1;
+			}
+			else
+			{
+				/* We were at the end of the string, so let's remove the slash
+				 * from the new string, too */
+				if (outpos > 1)
+					pTempName[outpos - 1] = 0;
+			}
 		}
 		else
 		{
@@ -645,7 +690,6 @@ void File_MakeAbsoluteName(char *pFileName)
 /**
  * Create a valid path name from a possibly invalid name by erasing invalid
  * path parts at the end of the string.
- * pPathName needs to point to a buffer of at least FILENAME_MAX bytes.
  */
 void File_MakeValidPathName(char *pPathName)
 {
@@ -666,7 +710,8 @@ void File_MakeValidPathName(char *pPathName)
 			/* Erase the (probably invalid) part after the last slash */
 			*pLastSlash = 0;
 		}
-		else
+		/* make sure we only shorten path... */
+		else if (pPathName[0])
 		{
 			/* Path name seems to be completely invalid -> set to root directory */
 			pPathName[0] = PATHSEP;
@@ -674,4 +719,67 @@ void File_MakeValidPathName(char *pPathName)
 		}
 	}
 	while (pLastSlash);
+
+	/* Make sure that path name ends with a slash */
+	File_AddSlashToEndFileName(pPathName);
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Remove given number of path elements from the end of the given path.
+ * Leaves '/' at the end if path still has directories. Given path
+ * may not be empty.
+ */
+void File_PathShorten(char *path, int dirs)
+{
+	int i, n = 0;
+	/* ignore last char, it may or may not be '/' */
+	i = strlen(path)-1;
+	assert(i >= 0);
+	while(i > 0 && n < dirs) {
+		if (path[--i] == PATHSEP)
+			n++;
+	}
+	if (path[i] == PATHSEP) {
+		path[i+1] = '\0';
+	} else {
+		path[0] = PATHSEP;
+		path[1] = '\0';
+	}
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  If "/." or "/.." at end, remove that and in case of ".." remove
+  also preceding dir (go one dir up).  Leave '/' at the end of
+  the path.
+*/
+void File_HandleDotDirs(char *path)
+{
+	int len = strlen(path);
+	if (len >= 2 &&
+	    path[len-2] == PATHSEP &&
+	    path[len-1] == '.')
+	{
+		/* keep in same dir */
+		path[len-1] = '\0';
+	}
+	else if (len >= 3 &&
+	    path[len-3] == PATHSEP &&
+	    path[len-2] == '.' &&
+	    path[len-1] == '.')
+	{
+		/* go one dir up */
+		if (len == 3) {
+			path[1] = 0;		/* already root */
+		} else {
+			char *ptr;
+			path[len-3] = 0;
+			ptr = strrchr(path, PATHSEP);
+			if (ptr)
+				*(ptr+1) = 0;
+		}
+	}
 }
