@@ -21,7 +21,7 @@
   (PaCifiST will, however, read/write to these images as it does not perform
   FDC access as on a real ST)
 */
-const char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.32 2006/08/08 07:19:15 thothy Exp $";
+const char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.35 2008/01/07 21:54:22 eerot Exp $";
 
 #include <sys/stat.h>
 
@@ -55,9 +55,9 @@ static const char * const pszDiskImageNameExts[] =
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Initialize emulation floppy drives
-*/
+/**
+ * Initialize emulation floppy drives
+ */
 void Floppy_Init(void)
 {
 	int i;
@@ -72,9 +72,9 @@ void Floppy_Init(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  UnInitialize drives
-*/
+/**
+ * UnInitialize drives
+ */
 void Floppy_UnInit(void)
 {
 	Floppy_EjectBothDrives();
@@ -82,9 +82,9 @@ void Floppy_UnInit(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
-*/
+/**
+ * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
+ */
 void Floppy_MemorySnapShot_Capture(BOOL bSave)
 {
 	int i;
@@ -115,9 +115,9 @@ void Floppy_MemorySnapShot_Capture(BOOL bSave)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Find which device to boot from (hard drive or floppy).
-*/
+/**
+ * Find which device to boot from (hard drive or floppy).
+ */
 void Floppy_GetBootDrive(void)
 {
 	/* Boot from hard drive if user wants this and HD emulation is turned on */
@@ -129,11 +129,11 @@ void Floppy_GetBootDrive(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Test if disk image is write protected. Write protection can be configured
-  in the GUI. When set to "automatic", we check the file permissions of the
-  floppy disk image to decide.
-*/
+/**
+ * Test if disk image is write protected. Write protection can be configured
+ * in the GUI. When set to "automatic", we check the file permissions of the
+ * floppy disk image to decide.
+ */
 BOOL Floppy_IsWriteProtected(int Drive)
 {
 	if (ConfigureParams.DiskImage.nWriteProtection == WRITEPROT_OFF)
@@ -157,14 +157,14 @@ BOOL Floppy_IsWriteProtected(int Drive)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Test disk image for valid boot-sector.
-  It has been noticed that some disks, eg blank images made by the MakeDisk
-  utility or PaCifiST emulator fill in the boot-sector with incorrect information.
-  Such images cannot be read correctly using a real ST, and also Hatari.
-  To try and prevent data loss, we check for this error and flag the drive so
-  the image will not be saved back to the file.
-*/
+/**
+ * Test disk image for valid boot-sector.
+ * It has been noticed that some disks, eg blank images made by the MakeDisk
+ * utility or PaCifiST emulator fill in the boot-sector with incorrect information.
+ * Such images cannot be read correctly using a real ST, and also Hatari.
+ * To try and prevent data loss, we check for this error and flag the drive so
+ * the image will not be saved back to the file.
+ */
 static BOOL Floppy_IsBootSectorOK(int Drive)
 {
 	Uint8 *pDiskBuffer;
@@ -192,15 +192,14 @@ static BOOL Floppy_IsBootSectorOK(int Drive)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Try to create disk B filename, eg 'auto_100a' becomes 'auto_100b'
-  Return TRUE if think we should try!
-*/
-static BOOL Floppy_CreateDiskBFileName(const char *pSrcFileName, char *pDestFileName)
+/**
+ * Try to create disk B filename, eg 'auto_100a' becomes 'auto_100b'
+ * Return new filename if think we should try, otherwise NULL
+ */
+static char* Floppy_CreateDiskBFileName(const char *pSrcFileName)
 {
 	char *szDir, *szName, *szExt;
-	BOOL bFileExists = FALSE;
-
+	
 	/* Allocate temporary memory for strings: */
 	szDir = malloc(3 * FILENAME_MAX);
 	if (!szDir)
@@ -212,7 +211,7 @@ static BOOL Floppy_CreateDiskBFileName(const char *pSrcFileName, char *pDestFile
 	szExt = szName + FILENAME_MAX;
 
 	/* So, first split name into parts */
-	File_splitpath(pSrcFileName, szDir, szName, szExt);
+	File_SplitPath(pSrcFileName, szDir, szName, szExt);
 
 	/* All OK? */
 	if (strlen(szName) > 0)
@@ -220,90 +219,116 @@ static BOOL Floppy_CreateDiskBFileName(const char *pSrcFileName, char *pDestFile
 		/* Now, did filename end with an 'A' or 'a'? */
 		if ((szName[strlen(szName)-1]=='A') || (szName[strlen(szName)-1]=='a'))
 		{
+			char *szFull;
 			/* Change 'A' to a 'B' */
 			szName[strlen(szName)-1] += 1;
 			/* And re-build name into destination */
-			File_makepath(pDestFileName, szDir, szName, szExt);
-			/* Does file exist? */
-			bFileExists = File_Exists(pDestFileName);
+			szFull = File_MakePath(szDir, szName, szExt);
+			if (szFull)
+			{
+				/* Does file exist? */
+				if (File_Exists(szFull))
+				{
+					free(szDir);
+					return szFull;
+				}
+				free(szFull);
+			}
 		}
 	}
-
 	free(szDir);
-
-	return bFileExists;
+	return NULL;
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Insert disk into floppy drive.
-  The WHOLE image is copied into our drive buffers, and uncompressed if necessary
-*/
-BOOL Floppy_InsertDiskIntoDrive(int Drive, char *pszFileName)
+/**
+ * Insert disk into floppy drive. Sets the disk image name
+ * back to given string with corrected extension. Maxlen is
+ * maximum string lenght.
+ * Returns TRUE of success and FALSE for failure.
+ * The WHOLE image is copied into our drive buffers,
+ * and uncompressed if necessary
+ */
+BOOL Floppy_InsertDiskIntoDrive(int Drive, char *pszFileName, int maxlen)
 {
-	return Floppy_ZipInsertDiskIntoDrive(Drive, pszFileName, NULL);
+	char *path;
+	path = Floppy_ZipInsertDiskIntoDrive(Drive, pszFileName, NULL);
+	if (path)
+	{
+		strncpy(pszFileName, path, maxlen);
+		pszFileName[maxlen-1] = '\0';
+		free(path);
+		return TRUE;
+	}
+	return FALSE;
 }
 
-BOOL Floppy_ZipInsertDiskIntoDrive(int Drive, char *pszFileName, char *pszZipPath)
+/* Read image into buffers, handles different image extensions,
+ * return corrected file name on success and NULL on failure.
+ */
+char* Floppy_ZipInsertDiskIntoDrive(int Drive, const char *pszFileName, const char *pszZipPath)
 {
 	long nImageBytes = 0;
+	char *filename;
 
-	/* Eject disk, if one is inserted(don't inform user) */
+	/* Eject disk, if one is inserted (doesn't inform user) */
 	Floppy_EjectDiskFromDrive(Drive,FALSE);
 
-	/* See if file exists, and if not get correct extension */
+	/* See if file exists, and if not, get/add correct extension */
 	if ( !File_Exists(pszFileName) )
-		File_FindPossibleExtFileName(pszFileName,pszDiskImageNameExts);
-
+		filename = File_FindPossibleExtFileName(pszFileName, pszDiskImageNameExts);
+	else
+		filename = strdup(pszFileName);
+	
 	/* Check disk image type and read the file: */
-	if (MSA_FileNameIsMSA(pszFileName, TRUE))
-		EmulationDrives[Drive].pBuffer = MSA_ReadDisk(pszFileName, &nImageBytes);
-	else if (ST_FileNameIsST(pszFileName, TRUE))
-		EmulationDrives[Drive].pBuffer = ST_ReadDisk(pszFileName, &nImageBytes);
-	else if (DIM_FileNameIsDIM(pszFileName, TRUE))
-		EmulationDrives[Drive].pBuffer = DIM_ReadDisk(pszFileName, &nImageBytes);
-	else if (ZIP_FileNameIsZIP(pszFileName))
-		EmulationDrives[Drive].pBuffer = ZIP_ReadDisk(pszFileName, pszZipPath, &nImageBytes);
+	if (MSA_FileNameIsMSA(filename, TRUE))
+		EmulationDrives[Drive].pBuffer = MSA_ReadDisk(filename, &nImageBytes);
+	else if (ST_FileNameIsST(filename, TRUE))
+		EmulationDrives[Drive].pBuffer = ST_ReadDisk(filename, &nImageBytes);
+	else if (DIM_FileNameIsDIM(filename, TRUE))
+		EmulationDrives[Drive].pBuffer = DIM_ReadDisk(filename, &nImageBytes);
+	else if (ZIP_FileNameIsZIP(filename))
+		EmulationDrives[Drive].pBuffer = ZIP_ReadDisk(filename, pszZipPath, &nImageBytes);
 
-	/* Did load OK? */
-	if (EmulationDrives[Drive].pBuffer != NULL)
+	if (EmulationDrives[Drive].pBuffer == NULL)
 	{
-		/* Store filename and size */
-		strcpy(EmulationDrives[Drive].szFileName,pszFileName);
-		EmulationDrives[Drive].nImageBytes = nImageBytes;
-		/* And set drive states */
-		EmulationDrives[Drive].bDiskInserted = TRUE;
-		EmulationDrives[Drive].bContentsChanged = FALSE;
-		EmulationDrives[Drive].bMediaChanged = TRUE;
-		EmulationDrives[Drive].bOKToSave = Floppy_IsBootSectorOK(Drive);
+		free(filename);
+		return NULL;
 	}
+	/* Store filename and size */
+	strcpy(EmulationDrives[Drive].szFileName, filename);
+	EmulationDrives[Drive].nImageBytes = nImageBytes;
+	/* And set drive states */
+	EmulationDrives[Drive].bDiskInserted = TRUE;
+	EmulationDrives[Drive].bContentsChanged = FALSE;
+	EmulationDrives[Drive].bMediaChanged = TRUE;
+	EmulationDrives[Drive].bOKToSave = Floppy_IsBootSectorOK(Drive);
 
-	/* If we insert a disk into Drive A, should be try to put disk 2 into drive B? */
+	/* If we insert a disk into Drive A, should we try to put disk 2 into drive B? */
 	if (Drive == 0 && ConfigureParams.DiskImage.bAutoInsertDiskB)
 	{
-		char *szDiskBFileName = malloc(FILENAME_MAX);
+		char *szTmp;
 		/* Attempt to make up second filename, eg was 'auto_100a' to 'auto_100b' */
-		if (szDiskBFileName && Floppy_CreateDiskBFileName(pszFileName, szDiskBFileName))
+		char *szDiskBFileName =  Floppy_CreateDiskBFileName(filename);
+		if (szDiskBFileName)
 		{
-			/* Put image into Drive B, clear out if fails */
-			if (!Floppy_InsertDiskIntoDrive(1,szDiskBFileName))
-				strcpy(EmulationDrives[1].szFileName,"");
+			/* recurse with Drive B */
+			szTmp = Floppy_ZipInsertDiskIntoDrive(1, szDiskBFileName, NULL);
+			if (szTmp)
+				free(szTmp);
+			free(szDiskBFileName);
 		}
-		free(szDiskBFileName);
 	}
-
-
-	/* Return TRUE if loaded OK */
-	return (EmulationDrives[Drive].pBuffer != NULL);
+	return filename;
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Eject disk from floppy drive, save contents back to PCs hard-drive if
-  they have been changed.
-*/
+/**
+ * Eject disk from floppy drive, save contents back to PCs hard-drive if
+ * they have been changed.
+ */
 void Floppy_EjectDiskFromDrive(int Drive, BOOL bInformUser)
 {
 	/* Does our drive have a disk in? */
@@ -349,9 +374,9 @@ void Floppy_EjectDiskFromDrive(int Drive, BOOL bInformUser)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Eject all disk image from floppy drives - call when quit.
-*/
+/**
+ * Eject all disk image from floppy drives - call when quit.
+ */
 void Floppy_EjectBothDrives(void)
 {
 	/* Eject disk images from drives 'A' and 'B' */
@@ -361,13 +386,13 @@ void Floppy_EjectBothDrives(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Double-check information read from boot-sector as this is sometimes found to
-  be incorrect. The .ST image file should be divisible by the sector size and
-  sectors per track.
-  NOTE - Pass information from boot-sector to this function (if we can't
-  decide we leave it alone).
-*/
+/**
+ * Double-check information read from boot-sector as this is sometimes found to
+ * be incorrect. The .ST image file should be divisible by the sector size and
+ * sectors per track.
+ * NOTE - Pass information from boot-sector to this function (if we can't
+ * decide we leave it alone).
+ */
 static void Floppy_DoubleCheckFormat(long nDiskSize, Uint16 *pnSides, Uint16 *pnSectorsPerTrack)
 {
 	int nSectorsPerTrack;
@@ -402,11 +427,11 @@ static void Floppy_DoubleCheckFormat(long nDiskSize, Uint16 *pnSides, Uint16 *pn
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Find details of disk image. We need to do this via a function as sometimes the boot-block
-  is not actually correct with the image - some demos/game disks have incorrect bytes in the
-  boot sector and this attempts to find the correct values.
-*/
+/**
+ * Find details of disk image. We need to do this via a function as sometimes the boot-block
+ * is not actually correct with the image - some demos/game disks have incorrect bytes in the
+ * boot sector and this attempts to find the correct values.
+ */
 void Floppy_FindDiskDetails(const Uint8 *pBuffer, int nImageBytes,
                             unsigned short *pnSectorsPerTrack, unsigned short *pnSides)
 {
@@ -432,10 +457,10 @@ void Floppy_FindDiskDetails(const Uint8 *pBuffer, int nImageBytes,
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read sectors from floppy disk image, return TRUE if all OK
-  NOTE Pass -ve as Count to read whole track
-*/
+/**
+ * Read sectors from floppy disk image, return TRUE if all OK
+ * NOTE Pass -ve as Count to read whole track
+ */
 BOOL Floppy_ReadSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
                         unsigned short Track, unsigned short Side, short Count,
                         int *pnSectorsPerTrack)
@@ -511,10 +536,10 @@ BOOL Floppy_ReadSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write sectors from floppy disk image, return TRUE if all OK
-  NOTE Pass -ve as Count to write whole track
-*/
+/**
+ * Write sectors from floppy disk image, return TRUE if all OK
+ * NOTE Pass -ve as Count to write whole track
+ */
 BOOL Floppy_WriteSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
                          unsigned short Track, unsigned short Side, short Count,
                          int *pnSectorsPerTrack)
