@@ -18,16 +18,22 @@
   * rmdir routine, can't remove dir with files in it. (another tos/unix difference)
   * Fix bugs, there are probably a few lurking around in here..
 */
-const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.60 2007/01/16 18:42:59 thothy Exp $";
+const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.69 2008/01/09 23:34:29 thothy Exp $";
+
+#include <config.h>
 
 #include <string.h>
-#include <strings.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <errno.h>
 
-#ifndef WIN32
+#if HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
+#if HAVE_GLOB_H
 #include <glob.h>
 #endif
 
@@ -48,11 +54,11 @@ const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.60 2007/01/16 18:42:59 tho
 #include "rs232.h"
 #include "scandir.h"
 #include "stMemory.h"
-#include "uae-cpu/hatari-glue.h"
-#include "uae-cpu/maccess.h"
+#include "hatari-glue.h"
+#include "maccess.h"
 
 
-// #define GEMDOS_VERBOSE
+//#define GEMDOS_VERBOSE
 // uncomment the following line to debug filename lookups on hd
 // #define GEMDOS_FILE_DEBUG 1
 
@@ -236,9 +242,9 @@ static const char *pszGemDOSNames[] =
 
 
 
-/* Poor Windows does not have a glob() function... */
-#ifdef WIN32
-#warning sorry, no glob function on Windows
+/* Poor Windows (and maybe other systems) do not have a glob() function... */
+#if !HAVE_GLOB_H
+#warning sorry, no glob function available
 
 typedef struct
 {
@@ -248,8 +254,8 @@ typedef struct
 } glob_t;
 
 static int glob(const char *pattern, int flags,
-	 int errfunc(const char *epath, int eerrno),
-         glob_t *pglob)
+                int errfunc(const char *epath, int eerrno),
+                glob_t *pglob)
 {
 	/* Just a quick hack to keep Hatari happy... */
 	pglob->gl_pathv = malloc(1 * sizeof(void *));
@@ -263,10 +269,18 @@ static void globfree(glob_t *pglob)
 	free(pglob->gl_pathv);
 }
 
-#define mkdir(name,mode) mkdir(name)
+#endif  /* HAVE_GLOB_H */
 
+
+#ifdef WIN32
+#define mkdir(name,mode) mkdir(name)
 #endif  /* WIN32 */
 
+#ifndef S_IRGRP
+#define S_IRGRP 0
+#define S_IROTH 0
+#endif
+
 
 
 /*-------------------------------------------------------*/
@@ -750,7 +764,8 @@ static int baselen(char *s)
 /**
  * Use hard-drive directory, current ST directory and filename to create full path
  */
-void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *pszDestName)
+void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName,
+                                    char *pszDestName, int nDestNameLen)
 {
 	char *s,*start;
 
@@ -763,22 +778,25 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 		return;
 
 	/* case full filename "C:\foo\bar" */
-	s=pszDestName;
-	start=NULL;
+	s = pszDestName;
+	start = NULL;
 
 	if (pszFileName[1] == ':')
 	{
-		sprintf(pszDestName, "%s%s", emudrives[0]->hd_emulation_dir, File_RemoveFileNameDrive(pszFileName));
+		snprintf(pszDestName, nDestNameLen, "%s%s",
+		         emudrives[0]->hd_emulation_dir, File_RemoveFileNameDrive(pszFileName));
 	}
 	/* case referenced from root:  "\foo\bar" */
 	else if (pszFileName[0] == '\\')
 	{
-		sprintf(pszDestName, "%s%s", emudrives[0]->hd_emulation_dir, pszFileName);
+		snprintf(pszDestName, nDestNameLen, "%s%s",
+		         emudrives[0]->hd_emulation_dir, pszFileName);
 	}
 	/* case referenced from current directory */
 	else
 	{
-		sprintf(pszDestName, "%s%s",  emudrives[0]->fs_currpath, pszFileName);
+		snprintf(pszDestName, nDestNameLen, "%s%s",
+		         emudrives[0]->fs_currpath, pszFileName);
 		start = pszDestName + strlen(emudrives[0]->fs_currpath)-1;
 	}
 
@@ -792,7 +810,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 		}
 		{
 			glob_t globbuf;
-			char old1,old2,dest[256];
+			char old1, old2;
 			int len, found, base_len;
 			unsigned int j;
 
@@ -817,8 +835,8 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 						strncasecmp(globbuf.gl_pathv[j],pszDestName,len)))
 				{
 					/* we found a matching name... */
-					sprintf(dest,"%s%c%s",globbuf.gl_pathv[j], PATHSEP,s+1);
-					strcpy(pszDestName,dest);
+					snprintf(pszDestName, nDestNameLen, "%s%c%s",
+					         globbuf.gl_pathv[j], PATHSEP, s+1);
 					j = globbuf.gl_pathc;
 					found = 1;
 				}
@@ -837,8 +855,8 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 					if (!strncasecmp(globbuf.gl_pathv[j],pszDestName,len))
 					{
 						/* we found a matching name... */
-						sprintf(dest,"%s%c%s",globbuf.gl_pathv[j], PATHSEP,s+1);
-						strcpy(pszDestName,dest);
+						snprintf(pszDestName, nDestNameLen, "%s%c%s",
+						         globbuf.gl_pathv[j], PATHSEP, s+1);
 						j = globbuf.gl_pathc;
 						found = 1;
 					}
@@ -887,7 +905,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 						strncasecmp(globbuf.gl_pathv[j],pszDestName,len)))
 				{
 					/* we found a matching name... */
-					strcpy(pszDestName,globbuf.gl_pathv[j]);
+					strncpy(pszDestName, globbuf.gl_pathv[j], nDestNameLen);
 					j = globbuf.gl_pathc;
 					found = 1;
 				}
@@ -1097,7 +1115,7 @@ static BOOL GemDOS_MkDir(Uint32 Params)
 		}
 
 		/* Copy old directory, as if calls fails keep this one */
-		GemDOS_CreateHardDriveFileName(Drive, pDirName, psDirPath);
+		GemDOS_CreateHardDriveFileName(Drive, pDirName, psDirPath, FILENAME_MAX);
 
 		/* Attempt to make directory */
 		if (mkdir(psDirPath, 0755) == 0)
@@ -1136,7 +1154,7 @@ static BOOL GemDOS_RmDir(Uint32 Params)
 		}
 
 		/* Copy old directory, as if calls fails keep this one */
-		GemDOS_CreateHardDriveFileName(Drive, pDirName, psDirPath);
+		GemDOS_CreateHardDriveFileName(Drive, pDirName, psDirPath, FILENAME_MAX);
 
 		/* Attempt to make directory */
 		if (rmdir(psDirPath) == 0)
@@ -1184,7 +1202,7 @@ static BOOL GemDOS_ChDir(Uint32 Params)
 			return TRUE;
 		}
 
-		GemDOS_CreateHardDriveFileName(Drive, pDirName, psTempDirPath);
+		GemDOS_CreateHardDriveFileName(Drive, pDirName, psTempDirPath, FILENAME_MAX);
 		if (stat(psTempDirPath, &buf))
 		{
 			/* error */
@@ -1215,58 +1233,78 @@ static BOOL GemDOS_ChDir(Uint32 Params)
 static BOOL GemDOS_Create(Uint32 Params)
 {
 	char szActualFileName[MAX_GEMDOS_PATH];
-	char *pszFileName;
-	const char *rwflags[] =
-		{
-			"w+", /* read / write (truncate if exists) */
-			"wb"  /* write only */
-		};
+	char *pszFileName, *ptr;
 	int Drive,Index,Mode;
+	const char *rwflags;
 
 	/* Find filename */
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
 	Mode = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG);
 	Drive = GemDOS_IsFileNameAHardDrive(pszFileName);
-	if (ISHARDDRIVE(Drive))
+	if (!ISHARDDRIVE(Drive))
 	{
-		/* And convert to hard drive filename */
-		GemDOS_CreateHardDriveFileName(Drive,pszFileName,szActualFileName);
-
-		/* Find slot to store file handle, as need to return WORD handle for ST */
-		Index = GemDOS_FindFreeFileHandle();
-		if (Index==-1)
-		{
-			/* No free handles, return error code */
-			Regs[REG_D0] = GEMDOS_ENHNDL;       /* No more handles */
-			return TRUE;
-		}
-		else
-		{
-#ifdef ENABLE_SAVING
-
-			FileHandles[Index].FileHandle = fopen(szActualFileName, rwflags[Mode&0x01]);
-
-			if (FileHandles[Index].FileHandle != NULL)
-			{
-				/* Tag handle table entry as used and return handle */
-				FileHandles[Index].bUsed = TRUE;
-				Regs[REG_D0] = Index+BASE_FILEHANDLE;  /* Return valid ST file handle from range 6 to 45! (ours start from 0) */
-				return TRUE;
-			}
-			else
-			{
-				Regs[REG_D0] = GEMDOS_EFILNF;     /* File not found */
-				return TRUE;
-			}
-#else
-			Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
-			return TRUE;
-#endif
-
-		}
+		return FALSE;
 	}
 
-	return FALSE;
+	if (Mode == GEMDOS_FILE_ATTRIB_VOLUME_LABEL)
+	{
+		fprintf(stderr, "Warning: Hatari doesn't support GEMDOS volume"
+		        " label setting\n(for '%s')\n", pszFileName);
+		Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
+		return TRUE;
+	}
+
+	/* Now convert to hard drive filename */
+	GemDOS_CreateHardDriveFileName(Drive, pszFileName,
+	                            szActualFileName, sizeof(szActualFileName));
+
+	/* Find slot to store file handle, as need to return WORD handle for ST */
+	Index = GemDOS_FindFreeFileHandle();
+	if (Index==-1)
+	{
+		/* No free handles, return error code */
+		Regs[REG_D0] = GEMDOS_ENHNDL;       /* No more handles */
+		return TRUE;
+	}
+#ifdef ENABLE_SAVING
+	/* FIXME: implement other Mode attributes
+	 * - GEMDOS_FILE_ATTRIB_HIDDEN       (FA_HIDDEN)
+	 * - GEMDOS_FILE_ATTRIB_SYSTEM_FILE  (FA_SYSTEM)
+	 * - GEMDOS_FILE_ATTRIB_SUBDIRECTORY (FA_DIR)
+	 * - GEMDOS_FILE_ATTRIB_WRITECLOSE   (FA_ARCHIVE)
+	 *   (set automatically by GemDOS >= 0.15)
+	 */
+	if (Mode & GEMDOS_FILE_ATTRIB_READONLY)
+		rwflags = "wb";
+	else
+		rwflags = "wb+";
+	FileHandles[Index].FileHandle = fopen(szActualFileName, rwflags);
+	
+	if (FileHandles[Index].FileHandle != NULL)
+	{
+		/* Tag handle table entry as used and return handle */
+		FileHandles[Index].bUsed = TRUE;
+		Regs[REG_D0] = Index+BASE_FILEHANDLE;  /* Return valid ST file handle from range 6 to 45! (ours start from 0) */
+		return TRUE;
+	}
+
+	/* We failed to create the file... now we have to return the right
+	 * error code: Normally we return FILE-NOT-FOUND, but in case the
+	 * directory did not exist yet, we have to return PATH-NOT-FOUND
+	 * (ST-Zip 2.6 relies on that during extraction of ZIP files). */
+	ptr = strrchr(szActualFileName, PATHSEP);
+	if (ptr)
+	{
+		*ptr = 0;   /* Strip filename from string */
+		if (!File_DirectoryExists(szActualFileName))
+		{
+			Regs[REG_D0] = GEMDOS_EPTHNF; /* Path not found */
+			return TRUE;
+		}
+	}
+#endif
+	Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
+	return TRUE;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -1279,9 +1317,12 @@ static BOOL GemDOS_Open(Uint32 Params)
 	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	const char *open_modes[] =
-		{ "rb", "wb", "r+", "rb"
-		}
-		;  /* convert atari modes to stdio modes */
+		{	/* convert atari modes to stdio modes */
+			"rb",	/* read only */
+			"rb+",	/* FIXME: should be write only, but "wb" truncates */
+			"rb+"	/* read/write */
+			"rb"	/* read only */
+		};
 	int Drive,Index,Mode;
 
 	/* Find filename */
@@ -1289,41 +1330,46 @@ static BOOL GemDOS_Open(Uint32 Params)
 	Mode = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG);
 	Drive = GemDOS_IsFileNameAHardDrive(pszFileName);
 
-	if (ISHARDDRIVE(Drive))
+	if (!ISHARDDRIVE(Drive))
 	{
-		/* And convert to hard drive filename */
-		GemDOS_CreateHardDriveFileName(Drive,pszFileName,szActualFileName);
-		/* Find slot to store file handle, as need to return WORD handle for ST  */
-		Index = GemDOS_FindFreeFileHandle();
-		if (Index == -1)
-		{
-			/* No free handles, return error code */
-			Regs[REG_D0] = GEMDOS_ENHNDL;       /* No more handles */
-			return TRUE;
-		}
+		return FALSE;
+	}
+	/* And convert to hard drive filename */
+	GemDOS_CreateHardDriveFileName(Drive, pszFileName,
+	                            szActualFileName, sizeof(szActualFileName));
 
-		/* Open file */
-		FileHandles[Index].FileHandle =  fopen(szActualFileName, open_modes[Mode&0x03]);
-
-		snprintf(FileHandles[Index].szActualName, sizeof(FileHandles[Index].szActualName),
-		         "%s", szActualFileName);
-
-		if (FileHandles[Index].FileHandle != NULL)
-		{
-			/* Tag handle table entry as used and return handle */
-			FileHandles[Index].bUsed = TRUE;
-			Regs[REG_D0] = Index+BASE_FILEHANDLE;  /* Return valid ST file handle from range 6 to 45! (ours start from 0) */
-			return TRUE;
-		}
-
-		if (Mode != 1 && errno == EACCES)
-			Log_Printf(LOG_DEBUG, "Missing permission to read file '%s'\n", szActualFileName);
-
-		Regs[REG_D0] = GEMDOS_EFILNF;     /* File not found/ error opening */
+	/* Find slot to store file handle, as need to return WORD handle for ST  */
+	Index = GemDOS_FindFreeFileHandle();
+	if (Index == -1)
+	{
+		/* No free handles, return error code */
+		Regs[REG_D0] = GEMDOS_ENHNDL;       /* No more handles */
 		return TRUE;
 	}
 
-	return FALSE;
+	/* FIXME: Open file
+	 * - fopen() modes don't allow write-only mode without truncating.
+	 *   Fixing this requires using open() and file descriptors instead
+	 *   of fopen() and FILE* pointers, but Windows doesn't support that
+	 */
+	FileHandles[Index].FileHandle =  fopen(szActualFileName, open_modes[Mode&0x03]);
+	
+	snprintf(FileHandles[Index].szActualName, sizeof(FileHandles[Index].szActualName),
+		 "%s", szActualFileName);
+	
+	if (FileHandles[Index].FileHandle != NULL)
+	{
+		/* Tag handle table entry as used and return handle */
+		FileHandles[Index].bUsed = TRUE;
+		Regs[REG_D0] = Index+BASE_FILEHANDLE;  /* Return valid ST file handle from range 6 to 45! (ours start from 0) */
+		return TRUE;
+	}
+	
+	if (Mode != 1 && errno == EACCES)
+		Log_Printf(LOG_DEBUG, "Missing permission to read file '%s'\n", szActualFileName);
+	
+	Regs[REG_D0] = GEMDOS_EFILNF;     /* File not found/ error opening */
+	return TRUE;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -1475,7 +1521,7 @@ static BOOL GemDOS_FDelete(Uint32 Params)
 		}
 
 		/* And convert to hard drive filename */
-		GemDOS_CreateHardDriveFileName(Drive, pszFileName, psActualFileName);
+		GemDOS_CreateHardDriveFileName(Drive, pszFileName, psActualFileName, FILENAME_MAX);
 
 		/* Now delete file?? */
 		if (unlink(psActualFileName) == 0)
@@ -1533,6 +1579,7 @@ static BOOL GemDOS_Fattrib(Uint32 Params)
 	char *psFileName;
 	int nDrive;
 	int nRwFlag, nAttrib;
+	struct stat FileStat;
 
 	/* Find filename */
 	psFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
@@ -1545,34 +1592,44 @@ static BOOL GemDOS_Fattrib(Uint32 Params)
 	Log_Printf(LOG_DEBUG, "Fattrib('%s', %d, 0x%x)\n", psFileName, nRwFlag, nAttrib);
 #endif
 
-	if (ISHARDDRIVE(nDrive))
+	if (!ISHARDDRIVE(nDrive))
 	{
-		struct stat FileStat;
+		return FALSE;
+	}
 
-		/* Convert to hard drive filename */
-		GemDOS_CreateHardDriveFileName(nDrive, psFileName, sActualFileName);
+	/* Convert to hard drive filename */
+	GemDOS_CreateHardDriveFileName(nDrive, psFileName,
+	                              sActualFileName, sizeof(sActualFileName));
 
-		if (stat(sActualFileName, &FileStat) != 0)
+	if (nAttrib == GEMDOS_FILE_ATTRIB_VOLUME_LABEL)
+	{
+		fprintf(stderr, "Warning: Hatari doesn't support GEMDOS volume label setting\n(for '%s')\n", sActualFileName);
+		Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
+		return TRUE;
+	}
+	if (stat(sActualFileName, &FileStat) != 0)
+	{
+		Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
+		return TRUE;
+	}
+	if (nRwFlag == 0)
+	{
+		/* Read attributes */
+		Regs[REG_D0] = GemDOS_ConvertAttribute(FileStat.st_mode);
+		return TRUE;
+	}
+	if (nAttrib & GEMDOS_FILE_ATTRIB_READONLY)
+	{
+		/* set read-only (readable by all) */
+		if (chmod(sActualFileName, S_IRUSR|S_IRGRP|S_IROTH) == 0)
 		{
-			Regs[REG_D0] = GEMDOS_EFILNF;         /* File not found */
-			return TRUE;
-		}
-
-		if (nRwFlag == 0)
-		{
-			/* Read attributes */
-			Regs[REG_D0] = GemDOS_ConvertAttribute(FileStat.st_mode);
-			return TRUE;
-		}
-		else
-		{
-			/* Write attributes */
-			Regs[REG_D0] = GEMDOS_EACCDN;         /* Acces denied */
+			Regs[REG_D0] = GEMDOS_FILE_ATTRIB_READONLY;
 			return TRUE;
 		}
 	}
-
-	return FALSE;
+	/* FIXME: support hidden/system/archive flags? */
+	Regs[REG_D0] = GEMDOS_EACCDN;         /* Acces denied */
+	return TRUE;
 }
 
 
@@ -1757,7 +1814,8 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 	{
 
 		/* And convert to hard drive filename */
-		GemDOS_CreateHardDriveFileName(Drive,pszFileName,szActualFileName);
+		GemDOS_CreateHardDriveFileName(Drive, pszFileName,
+		                    szActualFileName, sizeof(szActualFileName));
 
 		/* Populate DTA, set index for our use */
 		do_put_mem_word(pDTA->index, DTAIndex);
@@ -1856,8 +1914,10 @@ static BOOL GemDOS_Rename(Uint32 Params)
 	if (ISHARDDRIVE(NewDrive) && ISHARDDRIVE(OldDrive))
 	{
 		/* And convert to hard drive filenames */
-		GemDOS_CreateHardDriveFileName(NewDrive,pszNewFileName,szNewActualFileName);
-		GemDOS_CreateHardDriveFileName(OldDrive,pszOldFileName,szOldActualFileName);
+		GemDOS_CreateHardDriveFileName(NewDrive, pszNewFileName,
+		              szNewActualFileName, sizeof(szNewActualFileName));
+		GemDOS_CreateHardDriveFileName(OldDrive, pszOldFileName,
+		              szOldActualFileName, sizeof(szOldActualFileName));
 
 		/* Rename files */
 		if ( rename(szOldActualFileName,szNewActualFileName)==0 )
@@ -1926,17 +1986,18 @@ void GemDOS_OpCode(void)
 	unsigned short int GemDOSCall,CallingSReg;
 	Uint32 Params;
 	short RunOld;
+	Uint16 SR;
 
+	SR = M68000_GetSR();
 
 	/* Read SReg from stack to see if parameters are on User or Super stack  */
-	MakeSR();  /* update value of SR */
 	CallingSReg = STMemory_ReadWord(Regs[REG_A7]);
 	if ((CallingSReg&SR_SUPERMODE)==0)      /* Calling from user mode */
 		Params = regs.usp;
 	else
 	{
 		Params = Regs[REG_A7]+SIZE_WORD+SIZE_LONG;  /* super stack */
-		if (cpu_level > 0)
+		if (currprefs.cpu_level > 0)
 			Params += SIZE_WORD;   /* Skip extra word whe CPU is >=68010 */
 	}
 
@@ -2083,7 +2144,7 @@ void GemDOS_OpCode(void)
 		break;
 	}
 
-	MakeFromSR();  /* update the flags from the SR register */
+	M68000_SetSR(SR);   /* update the flags in the SR register */
 }
 
 
