@@ -79,6 +79,12 @@ enum
 };
 
 
+/* Commands are taking the equivalent of FDC_HBL_DELAY hbl's to execute */
+/* to try to simulate the speed of a real ST floppy drive */
+#define FDC_DELAY_CYCLES		92160
+//#define FDC_DELAY_CYCLES		1536	// 'Just Bugging Demo' by ACF requires a very fast delay (bug in the loader)
+
+
 extern Sint16 FDCSectorCountRegister;
 extern Uint16 DiskControllerWord_ff8604wr;
 extern Uint16 DMAModeControl_ff8606wr;
@@ -92,7 +98,7 @@ extern void FDC_DmaStatus_ReadWord(void);
 extern int FDC_FindFloppyDrive(void);
 extern void FDC_AcknowledgeInterrupt(void);
 extern void FDC_GpipRead(void);
-extern void FDC_UpdateHBL(void);
+extern void FDC_InterruptHandler_Update(void);
 extern void FDC_UpdateRestoreCmd(void);
 extern void FDC_UpdateSeekCmd(void);
 extern void FDC_UpdateStepCmd(void);
@@ -108,6 +114,8 @@ extern void FDC_DMADataFromFloppy(void);
 extern void FDC_DiskController_WriteWord(void);
 extern void FDC_DiskControllerStatus_ReadWord(void);
 extern void FDC_DmaModeControl_WriteWord(void);
+extern void FDC_FloppyMode_WriteByte(void);
+extern void FDC_FloppyMode_ReadByte(void);
 
 
 #endif /* ifndef HATARI_FDC_H */
