@@ -20,5 +20,12 @@ typedef struct
 extern const INTERCEPT_ACCESS_FUNC IoMemTable_ST[];
 extern const INTERCEPT_ACCESS_FUNC IoMemTable_STE[];
 extern const INTERCEPT_ACCESS_FUNC IoMemTable_TT[];
+extern const INTERCEPT_ACCESS_FUNC IoMemTable_Falcon[];
+
+extern void IoMemTabFalcon_DSPnone(void (**readtab)(void), void (**writetab)(void));
+extern void IoMemTabFalcon_DSPdummy(void (**readtab)(void), void (**writetab)(void));
+#if ENABLE_DSP_EMU
+extern void IoMemTabFalcon_DSPemulation(void (**readtab)(void), void (**writetab)(void));
+#endif
 
 #endif
