@@ -12,13 +12,9 @@
 #define  MAX_RS232INPUT_BUFFER    2048  /* Must be ^2 */
 
 extern BOOL bConnectedRS232;
-extern unsigned char InputBuffer_RS232[MAX_RS232INPUT_BUFFER];
-extern int InputBuffer_Head,InputBuffer_Tail;
 
 extern void RS232_Init(void);
 extern void RS232_UnInit(void);
-extern BOOL RS232_OpenCOMPort(void);
-extern void RS232_CloseCOMPort(void);
 extern void RS232_HandleUCR(short int ucr);
 extern BOOL RS232_SetBaudRate(int nBaud);
 extern void RS232_SetBaudRateFromTimerD(void);
@@ -26,7 +22,6 @@ extern void RS232_SetFlowControl(int ctrl);
 extern BOOL RS232_TransferBytesTo(unsigned char *pBytes, int nBytes);
 extern BOOL RS232_ReadBytes(unsigned char *pBytes, int nBytes);
 extern BOOL RS232_GetStatus(void);
-extern int RS232_ThreadFunc(void *pData);
 extern void RS232_SCR_ReadByte(void);
 extern void RS232_SCR_WriteByte(void);
 extern void RS232_UCR_ReadByte(void);
