@@ -11,16 +11,22 @@
 #include <SDL_video.h>    /* for SDL_Surface */
 
 
-/* Assumes 32 pixels left+right */
-#define SCREENBYTES_LEFT    16          /* Bytes for left border in ST screen */
-#define SCREENBYTES_MIDDLE  160         /* Middle (320 pixels) */
-#define SCREENBYTES_RIGHT   16          /* right border */
+/* The 'screen' is a representation of the ST video memory	*/
+/* taking into account all the border tricks. Data are stored	*/
+/* in 'planar' format (1 word per plane) and are then converted	*/
+/* to an SDL buffer that will be displayed.			*/
+/* So, all video lines are converted to a unique line of	*/
+/* SCREENBYTES_LINE bytes in planar format.			*/
+/* SCREENBYTES_LINE should always be a multiple of 8.		*/
+
+/* left/right borders must be multiple of 8 bytes */
+#define SCREENBYTES_LEFT    (nBorderPixelsLeft/2)  /* Bytes for left border */
+#define SCREENBYTES_MIDDLE  160                    /* Middle (320 pixels) */
+#define SCREENBYTES_RIGHT   (nBorderPixelsRight/2) /* Right border */
 #define SCREENBYTES_LINE    (SCREENBYTES_LEFT+SCREENBYTES_MIDDLE+SCREENBYTES_RIGHT)
 #define SCREENBYTES_MONOLINE 80         /* Byte per line in ST-high resolution */
 
 /* Overscan values */
-#define OVERSCAN_LEFT       (SCREENBYTES_LEFT*2)    /* Number of pixels in each border */
-#define OVERSCAN_RIGHT      (SCREENBYTES_RIGHT*2)
 #define OVERSCAN_TOP        29
 #define OVERSCAN_BOTTOM     47
 
@@ -43,14 +49,18 @@ typedef struct
 #define NUM_FRAMEBUFFERS  2
 
 
-/* ST Resolution defines */
+/* ST/TT resolution defines */
 enum
 {
   ST_LOW_RES,
   ST_MEDIUM_RES,
   ST_HIGH_RES,
-  ST_LOWMEDIUM_MIX_RES
+  TT_MEDIUM_RES = 4,
+  TT_HIGH_RES = 6,
+  TT_LOW_RES
 };
+#define ST_MEDIUM_RES_BIT 0x1
+#define ST_RES_MASK 0x3
 
 /* Update Palette defines */
 enum
@@ -91,7 +101,8 @@ enum
 
 extern BOOL bGrabMouse;
 extern BOOL bInFullScreen;
-extern int STRes,PrevSTRes;
+extern int nScreenZoomX, nScreenZoomY;
+extern int nBorderPixelsLeft, nBorderPixelsRight;
 extern int STScreenStartHorizLine;
 extern int STScreenLeftSkipBytes;
 extern FRAMEBUFFER *pFrameBuffer;
@@ -106,7 +117,7 @@ extern void Screen_Reset(void);
 extern void Screen_SetFullUpdate(void);
 extern void Screen_EnterFullScreen(void);
 extern void Screen_ReturnFromFullScreen(void);
-extern void Screen_DidResolutionChange(void);
+extern void Screen_ModeChanged(void);
 extern void Screen_Draw(void);
 
 #endif  /* ifndef HATARI_SCREEN_H */
