@@ -28,6 +28,7 @@ enum
 #define SG_TOUCHEXIT   1   /* Exit immediately when mouse button is pressed down */
 #define SG_EXIT        2   /* Exit when mouse button has been pressed (and released) */
 #define SG_DEFAULT     4   /* Marks a default button, selectable with return key */
+#define SG_CANCEL      8   /* Marks a cancel button, selectable with ESC key */
 
 /* Object states: */
 #define SG_SELECTED   1
@@ -64,6 +65,7 @@ extern int SDLGui_SetScreen(SDL_Surface *pScrn);
 extern void SDLGui_DrawDialog(const SGOBJ *dlg);
 extern int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut);
 extern void SDLGui_CenterDlg(SGOBJ *dlg);
-extern int SDLGui_FileSelect(char *path_and_name, char *zip_path, BOOL bAllowNew);
+extern char* SDLGui_FileSelect(const char *path_and_name, char **zip_path, BOOL bAllowNew);
+extern BOOL SDLGui_FileConfSelect(char *dlgname, char *confname, int maxlen, BOOL bAllowNew);
 
 #endif
