@@ -8,14 +8,21 @@
 #ifndef HATARI_STMEMORY_H
 #define HATARI_STMEMORY_H
 
+#include "config.h"
 #include "sysdeps.h"
 #include "maccess.h"
 #include "main.h"
 
-
+#if ENABLE_SMALL_MEM
+extern Uint8 *STRam;
+extern uae_u8 *ROMmemory;
+# define RomMem (ROMmemory-0xe00000)
+#else
 extern Uint8 STRam[16*1024*1024];
-extern Uint32 STRamEnd;
+#define RomMem STRam
+#endif  /* ENABLE_SMALL_MEM */
 
+extern Uint32 STRamEnd;
 
 /* Offset ST address to PC pointer: */
 #define STRAM_ADDR(Var)  ((unsigned long)STRam+((Uint32)(Var) & 0x00ffffff))
@@ -29,7 +36,14 @@ extern Uint32 STRamEnd;
 static inline void STMemory_WriteLong(Uint32 Address, Uint32 Var)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		do_put_mem_long(&ROMmemory[Address-0xe00000], Var);
+	else
+		do_put_mem_long(&STRam[Address], Var);
+#else
 	do_put_mem_long(&STRam[Address], Var);
+#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -40,7 +54,14 @@ static inline void STMemory_WriteLong(Uint32 Address, Uint32 Var)
 static inline void STMemory_WriteWord(Uint32 Address, Uint16 Var)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		do_put_mem_word(&ROMmemory[Address-0xe00000], Var);
+	else
+		do_put_mem_word(&STRam[Address], Var);
+#else
 	do_put_mem_word(&STRam[Address], Var);
+#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -50,7 +71,14 @@ static inline void STMemory_WriteWord(Uint32 Address, Uint16 Var)
 static inline void STMemory_WriteByte(Uint32 Address, Uint8 Var)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		ROMmemory[Address-0xe00000] = Var;
+	else
+		STRam[Address] = Var;
+#else
 	STRam[Address] = Var;
+#endif
 }
 
 
@@ -62,7 +90,14 @@ static inline void STMemory_WriteByte(Uint32 Address, Uint8 Var)
 static inline Uint32 STMemory_ReadLong(Uint32 Address)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		return do_get_mem_long(&ROMmemory[Address-0xe00000]);
+	else
+		return do_get_mem_long(&STRam[Address]);
+#else
 	return do_get_mem_long(&STRam[Address]);
+#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -73,7 +108,14 @@ static inline Uint32 STMemory_ReadLong(Uint32 Address)
 static inline Uint16 STMemory_ReadWord(Uint32 Address)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		return do_get_mem_word(&ROMmemory[Address-0xe00000]);
+	else
+		return do_get_mem_word(&STRam[Address]);
+#else
 	return do_get_mem_word(&STRam[Address]);
+#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -83,7 +125,14 @@ static inline Uint16 STMemory_ReadWord(Uint32 Address)
 static inline Uint8 STMemory_ReadByte(Uint32 Address)
 {
 	Address &= 0xffffff;
+#if ENABLE_SMALL_MEM
+	if (Address >= 0xe00000)
+		return ROMmemory[Address-0xe00000];
+	else
+		return STRam[Address];
+#else
 	return STRam[Address];
+#endif
 }
 
 
