@@ -12,6 +12,6 @@ extern BOOL bRecordingWav;
 
 extern BOOL WAVFormat_OpenFile(char *pszWavFileName);
 extern void WAVFormat_CloseFile(void);
-extern void WAVFormat_Update(char *pSamples, int Index, int Length);
+extern void WAVFormat_Update(Sint8 *pSamples, int Index, int Length);
 
 #endif
