@@ -16,13 +16,18 @@
   reduce redundancy and the function 'MemorySnapShot_Store' decides if it
   should save or restore the data.
 */
-const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.25 2006/08/22 18:39:23 thothy Exp $";
+const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.39 2008/03/29 23:01:56 thothy Exp $";
+
+#include <config.h>
 
 #include <SDL_types.h>
 #include <string.h>
-#include <strings.h>
 #include <errno.h>
 
+#if HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include "main.h"
 #include "blitter.h"
 #include "configuration.h"
@@ -33,6 +38,7 @@ const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.25 2006/08
 #include "gemdos.h"
 #include "ikbd.h"
 #include "int.h"
+#include "ioMem.h"
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
@@ -44,7 +50,7 @@ const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.25 2006/08
 #include "video.h"
 
 
-#define VERSION_STRING      "0.90 "   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define VERSION_STRING      "1.0.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
 #define VERSION_STRING_SIZE    6      /* Size of above (inc' NULL) */
 
 
@@ -67,9 +73,9 @@ static BOOL bCaptureSave, bCaptureError;
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Open file.
-*/
+/**
+ * Open file.
+ */
 static MSS_File MemorySnapShot_fopen(const char *pszFileName, const char *pszMode)
 {
 #ifdef COMPRESS_MEMORYSNAPSHOT
@@ -81,9 +87,9 @@ static MSS_File MemorySnapShot_fopen(const char *pszFileName, const char *pszMod
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Close file.
-*/
+/**
+ * Close file.
+ */
 static void MemorySnapShot_fclose(MSS_File fhndl)
 {
 #ifdef COMPRESS_MEMORYSNAPSHOT
@@ -95,9 +101,9 @@ static void MemorySnapShot_fclose(MSS_File fhndl)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read from file.
-*/
+/**
+ * Read from file.
+ */
 static int MemorySnapShot_fread(MSS_File fhndl, char *buf, int len)
 {
 #ifdef COMPRESS_MEMORYSNAPSHOT
@@ -109,9 +115,9 @@ static int MemorySnapShot_fread(MSS_File fhndl, char *buf, int len)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write data to file.
-*/
+/**
+ * Write data to file.
+ */
 static int MemorySnapShot_fwrite(MSS_File fhndl, const char *buf, int len)
 {
 #ifdef COMPRESS_MEMORYSNAPSHOT
@@ -123,10 +129,10 @@ static int MemorySnapShot_fwrite(MSS_File fhndl, const char *buf, int len)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Open/Create snapshot file, and set flag so 'MemorySnapShot_Store' knows
-  how to handle data.
-*/
+/**
+ * Open/Create snapshot file, and set flag so 'MemorySnapShot_Store' knows
+ * how to handle data.
+ */
 static BOOL MemorySnapShot_OpenFile(const char *pszFileName, BOOL bSave)
 {
 	char VersionString[VERSION_STRING_SIZE];
@@ -182,9 +188,9 @@ static BOOL MemorySnapShot_OpenFile(const char *pszFileName, BOOL bSave)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Close snapshot file.
-*/
+/**
+ * Close snapshot file.
+ */
 static void MemorySnapShot_CloseFile(void)
 {
 	MemorySnapShot_fclose(CaptureFile);
@@ -192,9 +198,9 @@ static void MemorySnapShot_CloseFile(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Save/Restore data to/from file.
-*/
+/**
+ * Save/Restore data to/from file.
+ */
 void MemorySnapShot_Store(void *pData, int Size)
 {
 	long nBytes;
@@ -216,15 +222,17 @@ void MemorySnapShot_Store(void *pData, int Size)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Save 'snapshot' of memory/chips/emulation variables
-*/
-void MemorySnapShot_Capture(const char *pszFileName)
+/**
+ * Save 'snapshot' of memory/chips/emulation variables
+ */
+void MemorySnapShot_Capture(const char *pszFileName, BOOL bConfirm)
 {
 	/* Set to 'saving' */
 	if (MemorySnapShot_OpenFile(pszFileName,TRUE))
 	{
 		/* Capture each files details */
+		Configuration_MemorySnapShot_Capture(TRUE);
+		TOS_MemorySnapShot_Capture(TRUE);
 		Main_MemorySnapShot_Capture(TRUE);
 		FDC_MemorySnapShot_Capture(TRUE);
 		Floppy_MemorySnapShot_Capture(TRUE);
@@ -235,7 +243,6 @@ void MemorySnapShot_Capture(const char *pszFileName)
 		MFP_MemorySnapShot_Capture(TRUE);
 		PSG_MemorySnapShot_Capture(TRUE);
 		Sound_MemorySnapShot_Capture(TRUE);
-		TOS_MemorySnapShot_Capture(TRUE);
 		Video_MemorySnapShot_Capture(TRUE);
 		Blitter_MemorySnapShot_Capture(TRUE);
 		DmaSnd_MemorySnapShot_Capture(TRUE);
@@ -247,21 +254,25 @@ void MemorySnapShot_Capture(const char *pszFileName)
 	/* Did error */
 	if (bCaptureError)
 		Log_AlertDlg(LOG_ERROR, "Unable to save memory state to file.");
-	else
+	else if (bConfirm)
 		Log_AlertDlg(LOG_INFO, "Memory state file saved.");
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Restore 'snapshot' of memory/chips/emulation variables
-*/
-void MemorySnapShot_Restore(const char *pszFileName)
+/**
+ * Restore 'snapshot' of memory/chips/emulation variables
+ */
+void MemorySnapShot_Restore(const char *pszFileName, BOOL bConfirm)
 {
 	/* Set to 'restore' */
 	if (MemorySnapShot_OpenFile(pszFileName,FALSE))
 	{
+		Configuration_MemorySnapShot_Capture(FALSE);
+		TOS_MemorySnapShot_Capture(FALSE);
+
 		/* Reset emulator to get things running */
+		IoMem_UnInit();  IoMem_Init();
 		Reset_Cold();
 
 		/* Capture each files details */
@@ -275,7 +286,6 @@ void MemorySnapShot_Restore(const char *pszFileName)
 		MFP_MemorySnapShot_Capture(FALSE);
 		PSG_MemorySnapShot_Capture(FALSE);
 		Sound_MemorySnapShot_Capture(FALSE);
-		TOS_MemorySnapShot_Capture(FALSE);
 		Video_MemorySnapShot_Capture(FALSE);
 		Blitter_MemorySnapShot_Capture(FALSE);
 		DmaSnd_MemorySnapShot_Capture(FALSE);
@@ -287,6 +297,38 @@ void MemorySnapShot_Restore(const char *pszFileName)
 	/* Did error? */
 	if (bCaptureError)
 		Log_AlertDlg(LOG_ERROR, "Unable to restore memory state from file.");
-	else
+	else if (bConfirm)
 		Log_AlertDlg(LOG_INFO, "Memory state file restored.");
 }
+
+
+/*-----------------------------------------------------------------------*/
+/*
+ * Save and restore functions required by the UAE CPU core...
+ * ... don't use them in normal Hatari code!
+ */
+#include <savestate.h>
+
+void save_u32(uae_u32 data)
+{
+	MemorySnapShot_Store(&data, 4);
+}
+
+void save_u16(uae_u16 data)
+{
+	MemorySnapShot_Store(&data, 2);
+}
+
+uae_u32 restore_u32(void)
+{
+	uae_u32 data;
+	MemorySnapShot_Store(&data, 4);
+	return data;
+}
+
+uae_u16 restore_u16(void)
+{
+	uae_u16 data;
+	MemorySnapShot_Store(&data, 2);
+	return data;
+}
