@@ -20,7 +20,7 @@
      the data register, but probably it should rather be done when reading the
      status register?).
 */
-const char Midi_rcsid[] = "Hatari $Id: midi.c,v 1.7 2006/02/08 22:49:27 eerot Exp $";
+const char Midi_rcsid[] = "Hatari $Id: midi.c,v 1.9 2007/02/25 21:20:10 eerot Exp $";
 
 #include <SDL_types.h>
 
@@ -30,13 +30,12 @@ const char Midi_rcsid[] = "Hatari $Id: midi.c,v 1.7 2006/02/08 22:49:27 eerot Ex
 #include "m68000.h"
 #include "mfp.h"
 #include "midi.h"
+#include "file.h"
 
 
 #define ACIA_SR_INTERRUPT_REQUEST  0x80
 
-
 #define MIDI_DEBUG 0
-
 #if MIDI_DEBUG
 #define Dprintf(a) printf a
 #else
@@ -50,49 +49,41 @@ static Uint8 MidiStatusRegister;
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Initialization: Open MIDI device.
-*/
+/**
+ * Initialization: Open MIDI device.
+ */
 void Midi_Init(void)
 {
 	MidiStatusRegister = 2;
 
-	if (ConfigureParams.Midi.bEnableMidi)
+	if (!ConfigureParams.Midi.bEnableMidi)
+		return;
+
+	/* Open MIDI file... */
+	pMidiOutFileHandle = File_Open(ConfigureParams.Midi.szMidiOutFileName, "wb");
+	if (!pMidiOutFileHandle)
 	{
-		/* Open MIDI file... */
-		pMidiOutFileHandle = fopen(ConfigureParams.Midi.szMidiOutFileName, "wb");
-
-		if (!pMidiOutFileHandle)
-		{
-			fprintf(stderr, "Failed to open %s.\n", ConfigureParams.Midi.szMidiOutFileName);
-			ConfigureParams.Midi.bEnableMidi = FALSE;
-			return;
-		}
-
-		Dprintf(("Opened midi file %s.\n", ConfigureParams.Midi.szMidiOutFileName));
+		ConfigureParams.Midi.bEnableMidi = FALSE;
+		return;
 	}
+	Dprintf(("Opened midi file '%s'.\n", ConfigureParams.Midi.szMidiOutFileName));
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Close MIDI device.
-*/
+/**
+ * Close MIDI device.
+ */
 void Midi_UnInit(void)
 {
-	if (pMidiOutFileHandle)
-	{
-		/* Close MIDI file... */
-		fclose(pMidiOutFileHandle);
-		pMidiOutFileHandle = NULL;
-	}
+	pMidiOutFileHandle = File_Close(pMidiOutFileHandle);
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read MIDI status register ($FFFC04).
-*/
+/**
+ * Read MIDI status register ($FFFC04).
+ */
 void Midi_Control_ReadByte(void)
 {
 	/* Dprintf(("Midi_ReadControl : $%x.\n", MidiStatusRegister)); */
@@ -105,9 +96,9 @@ void Midi_Control_ReadByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Read MIDI data register ($FFFC06).
-*/
+/**
+ * Read MIDI data register ($FFFC06).
+ */
 void Midi_Data_ReadByte(void)
 {
 	Dprintf(("Midi_ReadData : $%x.\n", 1));
@@ -122,9 +113,9 @@ void Midi_Data_ReadByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write to MIDI control register ($FFFC04).
-*/
+/**
+ * Write to MIDI control register ($FFFC04).
+ */
 void Midi_Control_WriteByte(void)
 {
 	/* ACIA registers need wait states - but the value seems to vary in certain cases */
@@ -148,9 +139,9 @@ void Midi_Control_WriteByte(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Write to MIDI data register ($FFFC06).
-*/
+/**
+ * Write to MIDI data register ($FFFC06).
+ */
 void Midi_Data_WriteByte(void)
 {
 	Uint8 dataByte;
