@@ -12,37 +12,46 @@
   NOTE - Tab's are converted to spaces as the PC 'Tab' setting differs to that
   of the ST.
 */
-const char Printer_rcsid[] = "Hatari $Id: printer.c,v 1.18 2006/02/08 22:49:27 eerot Exp $";
+const char Printer_rcsid[] = "Hatari $Id: printer.c,v 1.23 2008/02/23 16:51:27 thothy Exp $";
 
 #include "main.h"
-#include "dialog.h"
+#include "configuration.h"
 #include "file.h"
+#include "paths.h"
 #include "printer.h"
-#include "screen.h"
 
 /* #define PRINTER_DEBUG */
 
-#define PRINTER_FILENAME "/hatari.prn"
+#define PRINTER_FILENAME "hatari.prn"
 
 #define PRINTER_TAB_SETTING  8          /* A 'Tab' on the ST is 8 spaces */
 #define PRINTER_IDLE_CLOSE   (4*50)     /* After 4 seconds, close printer */
 
 #define PRINTER_BUFFER_SIZE  2048       /* 2k buffer which when full will be written to printer/file */
 
-static unsigned char PrinterBuffer[PRINTER_BUFFER_SIZE];   /* Buffer to store character before output */
+static Uint8 PrinterBuffer[PRINTER_BUFFER_SIZE];   /* Buffer to store character before output */
 static size_t nPrinterBufferChars;      /* # characters in above buffer */
 static int nPrinterBufferCharsOnLine;
-static BOOL bConnectedPrinter=FALSE;
-static BOOL bPrinterFile = FALSE;
+static BOOL bConnectedPrinter;
 static int nIdleCount;
 
-static FILE *PrinterFileHandle;
+static FILE *pPrinterHandle;
+
+
+/* internal functions */
+static void Printer_EmptyFile(void);
+static void Printer_ResetInternalBuffer(void);
+static void Printer_ResetCharsOnLine(void);
+static BOOL Printer_EmptyInternalBuffer(void);
+static BOOL Printer_ValidByte(Uint8 Byte);
+static void Printer_AddByteToInternalBuffer(Uint8 Byte);
+static void Printer_AddTabToInternalBuffer(void);
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Initialise Printer
-*/
+/**
+ * Initialise Printer
+ */
 void Printer_Init(void)
 {
 #ifdef PRINTER_DEBUG
@@ -51,15 +60,22 @@ void Printer_Init(void)
 
 	/* A valid file name for printing is already set up in configuration.c.
 	 * But we check it again since the user might have entered an invalid
-	 * file name in the ~/.hatari.cfg file... */
+	 * file name in the hatari.cfg file... */
 	if (strlen(ConfigureParams.Printer.szPrintToFileName) <= 1)
 	{
+		const char *psHomeDir;
+		psHomeDir = Paths_GetHatariHome();
+
 		/* construct filename for printing.... */
-		if (getenv("HOME") != NULL
-				&& strlen(getenv("HOME"))+strlen(PRINTER_FILENAME) < sizeof(ConfigureParams.Printer.szPrintToFileName))
-			sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%s", getenv("HOME"), PRINTER_FILENAME);
+		if (strlen(psHomeDir)+1+strlen(PRINTER_FILENAME) < sizeof(ConfigureParams.Printer.szPrintToFileName))
+		{
+			sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%c%s",
+			        psHomeDir, PATHSEP, PRINTER_FILENAME);
+		}
 		else
-			sprintf(ConfigureParams.Printer.szPrintToFileName, ".%s",PRINTER_FILENAME);
+		{
+			strcpy(ConfigureParams.Printer.szPrintToFileName, PRINTER_FILENAME);
+		}
 	}
 
 #ifdef PRINTER_DEBUG
@@ -69,9 +85,9 @@ void Printer_Init(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Uninitialise Printer
-*/
+/**
+ * Uninitialise Printer
+ */
 void Printer_UnInit(void)
 {
 	/* Close any open files */
@@ -84,16 +100,16 @@ void Printer_UnInit(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Close all open files etc.
-*/
+/**
+ * Close all open output file, empty buffers etc.
+ */
 void Printer_CloseAllConnections(void)
 {
 	/* Empty buffer */
 	Printer_EmptyInternalBuffer();
 
 	/* Close any open files */
-	Printer_CloseFile();
+	pPrinterHandle = File_Close(pPrinterHandle);
 
 	/* Signal finished with printing */
 	bConnectedPrinter = FALSE;
@@ -101,50 +117,36 @@ void Printer_CloseAllConnections(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Open file on disk, to which all printer output will be sent.
-*/
-BOOL Printer_OpenFile(void)
+/**
+ * Reset Printer Buffer
+ */
+static void Printer_ResetInternalBuffer(void)
 {
-
-	bPrinterFile = TRUE;
-
-	/* open printer file... */
-	PrinterFileHandle = fopen(ConfigureParams.Printer.szPrintToFileName, "a+");
-	if (!PrinterFileHandle)
-		bPrinterFile = FALSE;
-
-	return bPrinterFile;
+	nPrinterBufferChars = 0;
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Close file on disk, if we have one open.
-*/
-void Printer_CloseFile(void)
+/**
+ * Reset character line
+ */
+static void Printer_ResetCharsOnLine(void)
 {
-	/* Do have file open? */
-	if (bPrinterFile)
-	{
-		/* Close */
-		fclose(PrinterFileHandle);
-		bPrinterFile = FALSE;
-	}
+	nPrinterBufferCharsOnLine = 0;
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Empty to file on disk.
-*/
-void Printer_EmptyFile(void)
+/**
+ * Empty to file on disk.
+ */
+static void Printer_EmptyFile(void)
 {
 	/* Do have file open? */
-	if (bPrinterFile)
+	if (pPrinterHandle)
 	{
 		/* Write bytes out */
-		if (fwrite((unsigned char *)PrinterBuffer,sizeof(unsigned char),nPrinterBufferChars,PrinterFileHandle) < nPrinterBufferChars)
+		if (fwrite((Uint8 *)PrinterBuffer,sizeof(Uint8),nPrinterBufferChars,pPrinterHandle) < nPrinterBufferChars)
 		{
 			/* we wrote less then all chars in the buffer --> ERROR */
 			fprintf(stderr,"Printer_EmptyFile(): ERROR not all chars were written\n");
@@ -156,50 +158,29 @@ void Printer_EmptyFile(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Reset Printer Buffer
-*/
-void Printer_ResetInternalBuffer(void)
-{
-	nPrinterBufferChars = 0;
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Reset character line
-*/
-void Printer_ResetCharsOnLine(void)
-{
-	nPrinterBufferCharsOnLine = 0;
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Empty Printer Buffer
-*/
-BOOL Printer_EmptyInternalBuffer(void)
+/**
+ * Empty Printer Buffer
+ */
+static BOOL Printer_EmptyInternalBuffer(void)
 {
 	/* Write bytes to file */
-	if (nPrinterBufferChars>0)
+	if (nPrinterBufferChars > 0)
 	{
-		if (bPrinterFile)
+		if (pPrinterHandle)
 			Printer_EmptyFile();
 
 		return TRUE;
 	}
-
 	/* Nothing to do */
 	return FALSE;
 }
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Return TRUE if byte is standard ASCII character which is OK to output
-*/
-BOOL Printer_ValidByte(unsigned char Byte)
+/**
+ * Return TRUE if byte is standard ASCII character which is OK to output
+ */
+static BOOL Printer_ValidByte(Uint8 Byte)
 {
 	/* Return/New line? */
 	if ((Byte == 0x0d) || (Byte == 0x0a))
@@ -215,10 +196,10 @@ BOOL Printer_ValidByte(unsigned char Byte)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Add byte to our internal buffer, and when full write out - needed to speed
-*/
-void Printer_AddByteToInternalBuffer(unsigned char Byte)
+/**
+ * Add byte to our internal buffer, and when full write out - needed to speed
+ */
+static void Printer_AddByteToInternalBuffer(Uint8 Byte)
 {
 	/* Is buffer full? If so empty */
 	if (nPrinterBufferChars == PRINTER_BUFFER_SIZE)
@@ -232,10 +213,10 @@ void Printer_AddByteToInternalBuffer(unsigned char Byte)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Add 'Tab' to internal buffer
-*/
-void Printer_AddTabToInternalBuffer(void)
+/**
+ * Add 'Tab' to internal buffer
+ */
+static void Printer_AddTabToInternalBuffer(void)
 {
 	int i,NumSpaces;
 
@@ -253,10 +234,12 @@ void Printer_AddTabToInternalBuffer(void)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Pass byte from emulator to printer
-*/
-BOOL Printer_TransferByteTo(unsigned char Byte)
+/**
+ * Pass byte from emulator to printer.  Opens the printer file appending
+ * if it isn't already open. Returns FALSE if connection to "printer"
+ * failed
+ */
+BOOL Printer_TransferByteTo(Uint8 Byte)
 {
 	/* Do we want to output to a printer/file? */
 	if (!ConfigureParams.Printer.bEnablePrinting)
@@ -265,7 +248,9 @@ BOOL Printer_TransferByteTo(unsigned char Byte)
 	/* Have we made a connection to our printer/file? */
 	if (!bConnectedPrinter)
 	{
-		bConnectedPrinter = Printer_OpenFile();
+		/* open printer file... */
+		pPrinterHandle = File_Open(ConfigureParams.Printer.szPrintToFileName, "a+");
+		bConnectedPrinter = (pPrinterHandle != NULL);
 
 		/* Reset the printer */
 		Printer_ResetInternalBuffer();
@@ -294,10 +279,10 @@ BOOL Printer_TransferByteTo(unsigned char Byte)
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Empty printer buffer, and if remains idle for set time close connection
-  (ie close file, stop printer)
-*/
+/**
+ * Empty printer buffer, and if remains idle for set time close connection
+ * (ie close file, stop printer)
+ */
 void Printer_CheckIdleStatus(void)
 {
 	/* Is anything waiting for printer? */
