@@ -12,12 +12,12 @@
   In fact these mappings seems to force the gem to ask the IKBD for the real
   time (seconds units). See ikbd.c for the time returned by the IKBD.
 */
-const char Rtc_rcsid[] = "Hatari $Id: rtc.c,v 1.6 2007/01/16 18:42:59 thothy Exp $";
+const char Rtc_rcsid[] = "Hatari $Id: rtc.c,v 1.8 2007/12/19 11:54:41 thothy Exp $";
 
 #include <time.h>
 
 #include "main.h"
-#include "stMemory.h"
+#include "ioMem.h"
 #include "rtc.h"
 
 
@@ -31,13 +31,13 @@ static Sint8 fake_am, fake_amz;
  */
 void Rtc_SecondsUnits_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc21] = SystemTime->tm_sec % 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc21] = SystemTime->tm_sec % 10;
 }
 
 
@@ -47,13 +47,13 @@ void Rtc_SecondsUnits_ReadByte(void)
  */
 void Rtc_SecondsTens_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc23] = SystemTime->tm_sec / 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc23] = SystemTime->tm_sec / 10;
 }
 
 
@@ -63,20 +63,20 @@ void Rtc_SecondsTens_ReadByte(void)
  */
 void Rtc_MinutesUnits_ReadByte(void)
 {
-  if(rtc_bank)
-  {
-    STRam[0xfffc25] = fake_am;
-  }
-  else
-  {
-    struct tm *SystemTime;
-    time_t nTimeTicks;
+	if (rtc_bank)
+	{
+		IoMem[0xfffc25] = fake_am;
+	}
+	else
+	{
+		struct tm *SystemTime;
+		time_t nTimeTicks;
 
-    /* Get system time */
-    nTimeTicks = time(NULL);
-    SystemTime = localtime(&nTimeTicks);
-    STRam[0xfffc25] = SystemTime->tm_min % 10;
-  }
+		/* Get system time */
+		nTimeTicks = time(NULL);
+		SystemTime = localtime(&nTimeTicks);
+		IoMem[0xfffc25] = SystemTime->tm_min % 10;
+	}
 }
 
 
@@ -86,10 +86,10 @@ void Rtc_MinutesUnits_ReadByte(void)
  */
 void Rtc_MinutesUnits_WriteByte(void)
 {
-  /* TOS 1.0x uses this... */
-  if(rtc_bank)
-    fake_am = ((STRam[0xfffc25] & 0x0f) | 0xf0);
-  /* else ignore */
+	/* TOS 1.0x uses this... */
+	if (rtc_bank)
+		fake_am = ((IoMem[0xfffc25] & 0x0f) | 0xf0);
+	/* else ignore */
 }
 
 
@@ -99,20 +99,20 @@ void Rtc_MinutesUnits_WriteByte(void)
  */
 void Rtc_MinutesTens_ReadByte(void)
 {
-  if(rtc_bank)
-  {
-    STRam[0xfffc27] = fake_amz;
-  }
-  else
-  {
-    struct tm *SystemTime;
-    time_t nTimeTicks;
+	if (rtc_bank)
+	{
+		IoMem[0xfffc27] = fake_amz;
+	}
+	else
+	{
+		struct tm *SystemTime;
+		time_t nTimeTicks;
 
-    /* Get system time */
-    nTimeTicks = time(NULL);
-    SystemTime = localtime(&nTimeTicks);
-    STRam[0xfffc27] = SystemTime->tm_min / 10;
-  }
+		/* Get system time */
+		nTimeTicks = time(NULL);
+		SystemTime = localtime(&nTimeTicks);
+		IoMem[0xfffc27] = SystemTime->tm_min / 10;
+	}
 }
 
 
@@ -122,10 +122,10 @@ void Rtc_MinutesTens_ReadByte(void)
  */
 void Rtc_MinutesTens_WriteByte(void)
 {
-  /* TOS 1.0x uses this... */
-  if(rtc_bank)
-    fake_amz = ((STRam[0xfffc27] & 0x0f) | 0xf0);
-  /* else ignore */
+	/* TOS 1.0x uses this... */
+	if (rtc_bank)
+		fake_amz = ((IoMem[0xfffc27] & 0x0f) | 0xf0);
+	/* else ignore */
 }
 
 
@@ -135,13 +135,13 @@ void Rtc_MinutesTens_WriteByte(void)
  */
 void Rtc_HoursUnits_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc29] = SystemTime->tm_hour % 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc29] = SystemTime->tm_hour % 10;
 }
 
 
@@ -151,13 +151,13 @@ void Rtc_HoursUnits_ReadByte(void)
  */
 void Rtc_HoursTens_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc2b] = SystemTime->tm_hour / 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc2b] = SystemTime->tm_hour / 10;
 }
 
 
@@ -167,13 +167,13 @@ void Rtc_HoursTens_ReadByte(void)
  */
 void Rtc_Weekday_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc2d] = SystemTime->tm_wday;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc2d] = SystemTime->tm_wday;
 }
 
 
@@ -183,13 +183,13 @@ void Rtc_Weekday_ReadByte(void)
  */
 void Rtc_DayUnits_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc2f] = SystemTime->tm_mday % 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc2f] = SystemTime->tm_mday % 10;
 }
 
 
@@ -199,13 +199,13 @@ void Rtc_DayUnits_ReadByte(void)
  */
 void Rtc_DayTens_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc31] = SystemTime->tm_mday / 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc31] = SystemTime->tm_mday / 10;
 }
 
 
@@ -215,13 +215,13 @@ void Rtc_DayTens_ReadByte(void)
  */
 void Rtc_MonthUnits_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc33] = (SystemTime->tm_mon + 1) % 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc33] = (SystemTime->tm_mon + 1) % 10;
 }
 
 
@@ -231,13 +231,13 @@ void Rtc_MonthUnits_ReadByte(void)
  */
 void Rtc_MonthTens_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc35] = (SystemTime->tm_mon + 1) / 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc35] = (SystemTime->tm_mon + 1) / 10;
 }
 
 
@@ -247,13 +247,13 @@ void Rtc_MonthTens_ReadByte(void)
  */
 void Rtc_YearUnits_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc37] = SystemTime->tm_year % 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc37] = SystemTime->tm_year % 10;
 }
 
 
@@ -263,13 +263,13 @@ void Rtc_YearUnits_ReadByte(void)
  */
 void Rtc_YearTens_ReadByte(void)
 {
-  struct tm *SystemTime;
-  time_t nTimeTicks;
+	struct tm *SystemTime;
+	time_t nTimeTicks;
 
-  /* Get system time */
-  nTimeTicks = time(NULL);
-  SystemTime = localtime(&nTimeTicks);
-  STRam[0xfffc39] = (SystemTime->tm_year - 80) / 10;
+	/* Get system time */
+	nTimeTicks = time(NULL);
+	SystemTime = localtime(&nTimeTicks);
+	IoMem[0xfffc39] = (SystemTime->tm_year - 80) / 10;
 }
 
 
@@ -279,7 +279,7 @@ void Rtc_YearTens_ReadByte(void)
  */
 void Rtc_ClockMod_ReadByte(void)
 {
-  STRam[0xfffc3b] = ((STRam[0xfffc3b] & 0x0f) | 0xf0);
+	IoMem[0xfffc3b] = ((IoMem[0xfffc3b] & 0x0f) | 0xf0);
 }
 
 
@@ -289,6 +289,6 @@ void Rtc_ClockMod_ReadByte(void)
  */
 void Rtc_ClockMod_WriteByte(void)
 {
-  rtc_bank = STRam[0xfffc3b] & 1;
+	rtc_bank = IoMem[0xfffc3b] & 1;
 }
 
