@@ -15,7 +15,7 @@
   on boot-up which (correctly) cause a bus-error on Hatari as they would in a
   real STfm. If a user tries to select any of these images we bring up an error.
 */
-const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.50 2007/02/11 23:00:50 thothy Exp $";
+const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.54 2007/12/18 18:56:19 thothy Exp $";
 
 #include <SDL_endian.h>
 
@@ -31,7 +31,6 @@ const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.50 2007/02/11 23:00:50 thothy Ex
 #include "stMemory.h"
 #include "tos.h"
 #include "vdi.h"
-#include "uae-cpu/hatari-glue.h"
 
 
 Uint16 TosVersion;                      /* eg, 0x0100, 0x0102 */
@@ -224,7 +223,7 @@ static void TOS_FixRom(void)
 				{
 					/* Now we can really apply the patch! */
 					Log_Printf(LOG_DEBUG, "Applying TOS patch '%s'.\n", pPatch->pszName);
-					memcpy(&STRam[pPatch->Address], pPatch->pNewData, pPatch->Size);
+					memcpy(&RomMem[pPatch->Address], pPatch->pNewData, pPatch->Size);
 					nGoodPatches += 1;
 				}
 				else
@@ -275,7 +274,7 @@ static void TOS_CheckSysConfig(void)
 		ConfigureParams.System.nMachineType = MACHINE_TT;
 		IoMem_Init();
 		ConfigureParams.System.nCpuLevel = 3;
-		check_prefs_changed_cpu(ConfigureParams.System.nCpuLevel, ConfigureParams.System.bCompatibleCpu);
+		M68000_CheckCpuLevel();
 	}
 	else if ((TosVersion & 0x0f00) == 0x0400 && ConfigureParams.System.nMachineType != MACHINE_FALCON)
 	{
@@ -285,7 +284,7 @@ static void TOS_CheckSysConfig(void)
 		ConfigureParams.System.nMachineType = MACHINE_FALCON;
 		IoMem_Init();
 		ConfigureParams.System.nCpuLevel = 3;
-		check_prefs_changed_cpu(ConfigureParams.System.nCpuLevel, ConfigureParams.System.bCompatibleCpu);
+		M68000_CheckCpuLevel();
 	}
 	else if ((TosVersion < 0x0300 && ConfigureParams.System.nMachineType == MACHINE_FALCON)
 	         || (TosVersion < 0x0200 && ConfigureParams.System.nMachineType == MACHINE_TT))
@@ -298,7 +297,7 @@ static void TOS_CheckSysConfig(void)
 		if (TosVersion <= 0x0104)
 		{
 			ConfigureParams.System.nCpuLevel = 0;
-			check_prefs_changed_cpu(ConfigureParams.System.nCpuLevel, ConfigureParams.System.bCompatibleCpu);
+			M68000_CheckCpuLevel();
 		}
 	}
 }
@@ -319,7 +318,7 @@ int TOS_LoadImage(void)
 
 	/* Load TOS image into memory so we can check it's vesion */
 	TosVersion = 0;
-	pTosFile = File_Read(ConfigureParams.Rom.szTosImageFileName, NULL, &nFileSize, pszTosNameExts);
+	pTosFile = File_Read(ConfigureParams.Rom.szTosImageFileName, &nFileSize, pszTosNameExts);
 
 	if (!pTosFile || nFileSize <= 0)
 	{
@@ -371,8 +370,22 @@ int TOS_LoadImage(void)
 	if (!bIsEmuTOS)
 		TOS_CheckSysConfig();
 
+	/* Calculate end of RAM */
+	if (ConfigureParams.Memory.nMemorySize > 0
+	    && ConfigureParams.Memory.nMemorySize <= 14)
+		STRamEnd = ConfigureParams.Memory.nMemorySize * 0x100000;
+	else
+		STRamEnd = 0x80000;   /* 512 KiB */
+
+	/* (Re-)Initialize the memory banks: */
+	memory_uninit();
+	memory_init(STRamEnd, 0, TosAddress);
+
+	/* Clear Upper memory (ROM and IO memory) */
+	memset(&RomMem[0xe00000], 0, 0x200000);
+
 	/* Copy loaded image into ST memory */
-	memcpy(STRam+TosAddress, pTosFile, TosSize);
+	memcpy(&RomMem[TosAddress], pTosFile, TosSize);
 
 	Log_Printf(LOG_DEBUG, "Loaded TOS version %i.%c%c, starting at $%x, "
 	           "country code = %i, %s\n", TosVersion>>8, '0'+((TosVersion>>4)&0x0f),
