@@ -8,22 +8,16 @@
 #ifndef HATARI_GLUE_H
 #define HATARI_GLUE_H
 
-
 #include "sysdeps.h"
+#include "options_cpu.h"
+#include "../includes/cycles.h"
 
-
-extern int illegal_mem;
-extern int address_space_24;
-extern int cpu_level;
-extern int cpu_compatible;
 extern int pendingInterrupts;
 
 int Init680x0(void);
 void Exit680x0(void);
-void Start680x0(void);
 void customreset(void);
 int intlev (void);
-void check_prefs_changed_cpu(int new_level, int new_compatible);
 
 unsigned long OpCode_GemDos(uae_u32 opcode);
 unsigned long OpCode_SysInit(uae_u32 opcode);
