@@ -10,14 +10,16 @@
   * This file is distributed under the GNU Public License, version 2 or at
   * your option any later version. Read the file gpl.txt for details.
   */
-const char Memory_rcsid[] = "Hatari $Id: memory.c,v 1.21 2006/02/21 21:45:32 thothy Exp $";
+const char Memory_rcsid[] = "Hatari $Id: memory.c,v 1.24 2007/12/30 20:02:04 thothy Exp $";
 
+#include "config.h"
 #include "sysdeps.h"
 #include "hatari-glue.h"
 #include "maccess.h"
 #include "memory.h"
 #include "../includes/main.h"
 #include "../includes/tos.h"
+#include "../includes/ide.h"
 #include "../includes/ioMem.h"
 #include "../includes/reset.h"
 #include "../includes/stMemory.h"
@@ -25,19 +27,26 @@ const char Memory_rcsid[] = "Hatari $Id: memory.c,v 1.21 2006/02/21 21:45:32 tho
 #include "newcpu.h"
 
 
+/* Set illegal_mem to 1 for debug output: */
+#define illegal_mem 1
+
+
 static uae_u32 STmem_size, TTmem_size = 0;
 static uae_u32 TTmem_mask;
 
 #define STmem_start  0x00000000
 #define ROMmem_start 0x00E00000
+#define IdeMem_start 0x00F00000
 #define IOmem_start  0x00FF0000
 #define TTmem_start  0x01000000
 
+#define IdeMem_size  65536
 #define IOmem_size  65536
 #define ROMmem_size (0x00FF0000 - 0x00E00000)  /* So we cover both possible ROM regions + cartridge */
 
 #define STmem_mask  0x00ffffff
 #define ROMmem_mask 0x00ffffff
+#define IdeMem_mask  (IdeMem_size - 1)
 #define IOmem_mask  (IOmem_size - 1)
 
 
@@ -477,7 +486,7 @@ static uae_u8 *TTmem_xlate(uaecptr addr)
 
 /* **** ROM memory **** */
 
-static uae_u8 *ROMmemory;
+uae_u8 *ROMmemory;
 
 static uae_u32 ROMmem_lget(uaecptr addr)
 {
@@ -539,10 +548,30 @@ static uae_u8 *ROMmem_xlate(uaecptr addr)
 }
 
 
-/* Hardware IO memory */
-/* see also intercept.c */
+/* IDE controller IO memory */
+/* see also ide.c */
 
-static uae_u8 *IOmemory;
+static uae_u8 *IdeMemory;
+
+static int IdeMem_check(uaecptr addr, uae_u32 size)
+{
+    addr -= IdeMem_start;
+    addr &= IdeMem_mask;
+    return (addr + size) <= IdeMem_size;
+}
+
+static uae_u8 *IdeMem_xlate(uaecptr addr)
+{
+    addr -= IdeMem_start;
+    addr &= IdeMem_mask;
+    return IdeMemory + addr;
+}
+
+
+/* Hardware IO memory */
+/* see also ioMem.c */
+
+uae_u8 *IOmemory;
 
 static int IOmem_check(uaecptr addr, uae_u32 size)
 {
@@ -611,6 +640,13 @@ static addrbank ROMmem_bank =
     ROMmem_xlate, ROMmem_check
 };
 
+static addrbank IdeMem_bank =
+{
+    Ide_Mem_lget, Ide_Mem_wget, Ide_Mem_bget,
+    Ide_Mem_lput, Ide_Mem_wput, Ide_Mem_bput,
+    IdeMem_xlate, IdeMem_check
+};
+
 static addrbank IOmem_bank =
 {
     IoMem_lget, IoMem_wget, IoMem_bget,
@@ -639,24 +675,39 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     /*write_log("memory_init: STmem_size=$%x, TTmem_size=$%x, ROM-Start=$%x,\n",
               STmem_size, TTmem_size, nNewRomMemStart);*/
 
-/*  // STmemory is currently directly defined to STRam for speed reasons!
-    STmemory = STRam;
-*/
-    ROMmemory = STRam + ROMmem_start;
-    IOmemory = STRam + IOmem_start;
+#if ENABLE_SMALL_MEM
 
-/*
-    while (! STmemory && STmem_size > 512*1024) {
+    /* Allocate memory for ROM areas and IO memory space (0xE00000 - 0xFFFFFF) */
+    ROMmemory = malloc(2*1024*1024);
+    if (!ROMmemory) {
+	fprintf(stderr, "Out of memory (ROM/IO mem)!\n");
+	exit(1);
+    }
+    IdeMemory = ROMmemory + 0x100000;
+    IOmemory  = ROMmemory + 0x1f0000;
+
+    /* Allocate memory for normal ST RAM */
+    STmemory = malloc(STmem_size);
+    while (!STmemory && STmem_size > 512*1024) {
 	STmem_size >>= 1;
 	STmemory = (uae_u8 *)malloc (STmem_size);
 	if (STmemory)
-	    fprintf (stderr, "Reducing STmem size to %dkb\n", STmem_size >> 10);
+	    write_log ("Reducing STmem size to %dkb\n", STmem_size >> 10);
     }
-    if (! STmemory) {
+    if (!STmemory) {
 	write_log ("virtual memory exhausted (STmemory)!\n");
-	abort ();
+	exit(1);
     }
-*/
+
+#else
+
+    /* STmemory points to the 16 MiB STRam array, we just have to set up
+     * the remaining pointers here: */
+    ROMmemory = STRam + ROMmem_start;
+    IdeMemory = STRam + IdeMem_start;
+    IOmemory = STRam + IOmem_start;
+
+#endif
 
     init_mem_banks();
 
@@ -701,8 +752,11 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     /* IO memory: */
     map_banks(&IOmem_bank, IOmem_start>>16, 0x1);
 
+    /* IDE controller memory region: */
+    map_banks(&IdeMem_bank, IdeMem_start >> 16, 0x1);  /* IDE controller on the Falcon */
+
     /* Illegal memory regions cause a bus error on the ST: */
-    map_banks(&BusErrMem_bank, 0xF00000 >> 16, 0xA);  /* IDE controller on the Falcon */
+    map_banks(&BusErrMem_bank, 0xF10000 >> 16, 0x9);
 }
 
 
@@ -712,11 +766,24 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
 void memory_uninit (void)
 {
     /* Here, we free allocated memory from memory_init */
-    if(TTmem_size > 0)
-    {
-      free(TTmemory);
-      TTmemory = NULL;
+    if (TTmem_size > 0) {
+	free(TTmemory);
+	TTmemory = NULL;
     }
+
+#if ENABLE_SMALL_MEM
+
+    if (STmemory) {
+	free(STmemory);
+	STmemory = NULL;
+    }
+
+    if (ROMmemory) {
+	free(ROMmemory);
+	ROMmemory = NULL;
+    }
+
+#endif  /* ENABLE_SMALL_MEM */
 }
 
 
@@ -731,7 +798,7 @@ void map_banks (addrbank *bank, int start, int size)
 	return;
     }
     /* Some ROMs apparently require a 24 bit address space... */
-    if (address_space_24)
+    if (currprefs.address_space_24)
 	endhioffs = 0x10000;
     for (hioffs = 0; hioffs < endhioffs; hioffs += 0x100)
 	for (bnr = start; bnr < start+size; bnr++)
