@@ -11,16 +11,20 @@
 
    Read unzip.h for more info
 */
-const char Unzip_rcsid[] = "Hatari $Id: unzip.c,v 1.10 2007/01/16 18:42:59 thothy Exp $";
+const char Unzip_rcsid[] = "Hatari $Id: unzip.c,v 1.12 2007/12/18 20:35:06 thothy Exp $";
 
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
-#include <strings.h>
 #include <zlib.h>
 
+#if HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include "unzip.h"
 
 #ifdef NO_ERRNO_H
@@ -165,12 +169,12 @@ local int unzlocal_getByte(FILE *fin, int *pi)
 local int unzlocal_getShort (FILE* fin, uLong *pX)
 {
     uLong x ;
-    int i;
+    int i = 0;
     int err;
 
     err = unzlocal_getByte(fin,&i);
     x = (uLong)i;
-    
+
     if (err==UNZ_OK)
         err = unzlocal_getByte(fin,&i);
     x += ((uLong)i)<<8;
@@ -185,12 +189,12 @@ local int unzlocal_getShort (FILE* fin, uLong *pX)
 local int unzlocal_getLong (FILE* fin, uLong *pX)
 {
     uLong x ;
-    int i;
+    int i = 0;
     int err;
 
     err = unzlocal_getByte(fin,&i);
     x = (uLong)i;
-    
+
     if (err==UNZ_OK)
         err = unzlocal_getByte(fin,&i);
     x += ((uLong)i)<<8;
@@ -202,7 +206,7 @@ local int unzlocal_getLong (FILE* fin, uLong *pX)
     if (err==UNZ_OK)
         err = unzlocal_getByte(fin,&i);
     x += ((uLong)i)<<24;
-   
+
     if (err==UNZ_OK)
         *pX = x;
     else
