@@ -6,7 +6,7 @@
 
   YM File output, for use with STSound etc...
 */
-const char YMFormat_rcsid[] = "Hatari $Id: ymFormat.c,v 1.15 2007/01/16 18:42:59 thothy Exp $";
+const char YMFormat_rcsid[] = "Hatari $Id: ymFormat.c,v 1.17 2007/10/31 21:31:50 eerot Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -21,32 +21,39 @@ const char YMFormat_rcsid[] = "Hatari $Id: ymFormat.c,v 1.15 2007/01/16 18:42:59
 #define YM_RECORDSIZE  (4+(YM_MAX_VBLS*NUM_PSG_SOUND_REGISTERS))  /* ~330k for 8 minutes */
 
 BOOL bRecordingYM = FALSE;
-int nYMVBLS = 0;
-unsigned char *pYMWorkspace = NULL, *pYMData;
-
+static int nYMVBLS = 0;
+static Uint8 *pYMData, *pYMWorkspace = NULL;
+static char *pszYMFileName = NULL;
 
 /*-----------------------------------------------------------------------*/
 /**
  * Start recording YM registers to workspace
  */
-BOOL YMFormat_BeginRecording(char *pszYMFileName)
+BOOL YMFormat_BeginRecording(const char *filename)
 {
 	/* Free any previous data, don't save */
-	YMFormat_FreeRecording();
+	bRecordingYM = FALSE;
+	YMFormat_EndRecording();
 
 	/* Make sure we have a proper filename to use */
-	if (!pszYMFileName || strlen(pszYMFileName)<=0)
+	if (!filename || strlen(filename) <= 0)
+	{
+		return FALSE;
+	}
+	pszYMFileName = strdup(filename);
+	if (!pszYMFileName)
 	{
 		return FALSE;
 	}
 
 	/* Create YM workspace */
-	pYMWorkspace = (unsigned char *)malloc(YM_RECORDSIZE);
+	pYMWorkspace = (Uint8 *)malloc(YM_RECORDSIZE);
 
 	if (!pYMWorkspace)
 	{
 		/* Failed to allocate memory, cannot record */
-		bRecordingYM = FALSE;
+		free(pszYMFileName);
+		pszYMFileName = NULL;
 		return FALSE;
 	}
 
@@ -69,42 +76,92 @@ BOOL YMFormat_BeginRecording(char *pszYMFileName)
 
 /*-----------------------------------------------------------------------*/
 /**
- * End recording YM registers and save as '.YM' file
+ * Convert YM data to stream for output
+ *
+ * Data is:
+ *   4 Byte header 'YM3!'
+ *   VBL Count x 14 PSG registers
+ * BUT
+ *   We need data in a register stream, eg Reg 0, VBL 1, VBL 2, VBL n and then next register...
+ * 
+ * Convert to new workspace and return TRUE if all OK
  */
-void YMFormat_EndRecording(void)
+static BOOL YMFormat_ConvertToStreams(void)
 {
-	/* Have recorded information? */
-	if (pYMWorkspace && nYMVBLS)
+	Uint8 *pNewYMWorkspace;
+	Uint8 *pTmpYMData, *pNewYMData;
+	Uint8 *pTmpYMStream, *pNewYMStream;
+	int Reg, Count;
+
+	/* Allocate new workspace to convert data to */
+	pNewYMWorkspace = (Uint8 *)malloc(YM_RECORDSIZE);
+	if (pNewYMWorkspace)
 	{
-		/* Convert YM to correct format(list of register 1, then register 2...) */
-		if (YMFormat_ConvertToStreams())
+		/* Convert data, first copy over header */
+		pTmpYMData = pYMWorkspace;
+		pNewYMData = pNewYMWorkspace;
+		*pNewYMData++ = *pTmpYMData++;
+		*pNewYMData++ = *pTmpYMData++;
+		*pNewYMData++ = *pTmpYMData++;
+		*pNewYMData++ = *pTmpYMData++;
+
+		/* Now copy over each stream */
+		for(Reg=0; Reg<NUM_PSG_SOUND_REGISTERS; Reg++)
 		{
-			/* Save YM File */
-			if (strlen(ConfigureParams.Sound.szYMCaptureFileName) > 0)
+			/* Get pointer to source / destination */
+			pTmpYMStream = pTmpYMData + Reg;
+			pNewYMStream = pNewYMData + (Reg*nYMVBLS);
+
+			/* Copy recording VBLs worth */
+			for(Count=0; Count<nYMVBLS; Count++)
 			{
-				File_Save(ConfigureParams.Sound.szYMCaptureFileName, pYMWorkspace,(size_t)(nYMVBLS*NUM_PSG_SOUND_REGISTERS)+4, FALSE);
-				/* And inform user */
-				Log_AlertDlg(LOG_INFO, "YM sound data recording has been stopped.");
+				*pNewYMStream++ = *pTmpYMStream;
+				pTmpYMStream += NUM_PSG_SOUND_REGISTERS;
 			}
 		}
-	}
 
-	/* And free */
-	YMFormat_FreeRecording();
+		/* Delete old workspace and assign new */
+		free(pYMWorkspace);
+		pYMWorkspace = pNewYMWorkspace;
+
+		return TRUE;
+	}
+	return FALSE;
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Free up any resources used by YM recording
+ * End recording YM registers and save as '.YM' file
  */
-void YMFormat_FreeRecording(void)
+void YMFormat_EndRecording(void)
 {
-	/* Free workspace */
-	if (pYMWorkspace)
-		free(pYMWorkspace);
-	pYMWorkspace = NULL;
+	/* Recording, have recorded information? */
+	if (bRecordingYM && pszYMFileName && pYMWorkspace && nYMVBLS)
+	{
+		/* Convert YM to correct format(list of register 1, then register 2...) */
+		if (YMFormat_ConvertToStreams())
+		{
+			/* Save YM File */
+			File_Save(pszYMFileName, pYMWorkspace,(size_t)(nYMVBLS*NUM_PSG_SOUND_REGISTERS)+4, FALSE);
+			/* And inform user */
+			Log_AlertDlg(LOG_INFO, "YM sound data recording has been stopped.");
+		}
+		else
+			Log_AlertDlg(LOG_INFO, "YM sound data conversion failed!");
 
+	}
+	/* And free */
+	if (pYMWorkspace)
+	{
+		free(pYMWorkspace);
+		pYMWorkspace = NULL;
+	}
+	if (pszYMFileName)
+	{
+		free(pszYMFileName);
+		pszYMFileName = NULL;
+	}
 	/* Stop recording */
 	bRecordingYM = FALSE;
 }
@@ -138,59 +195,3 @@ void YMFormat_UpdateRecording(void)
 	}
 }
 
-
-/*-----------------------------------------------------------------------*/
-/**
- * Convert YM data to stream for output
- *
- * Data is:
- *   4 Byte header 'YM3!'
- *   VBL Count x 14 PSG registers
- * BUT
- *   We need data in a register stream, eg Reg 0, VBL 1, VBL 2, VBL n and then next register...
- * 
- * Convert to new workspace and return TRUE if all OK
- */
-BOOL YMFormat_ConvertToStreams(void)
-{
-	unsigned char *pNewYMWorkspace;
-	unsigned char *pTmpYMData, *pNewYMData;
-	unsigned char *pTmpYMStream, *pNewYMStream;
-	int Reg, Count;
-
-	/* Allocate new workspace to convert data to */
-	pNewYMWorkspace = (unsigned char *)malloc(YM_RECORDSIZE);
-	if (pNewYMWorkspace)
-	{
-		/* Convert data, first copy over header */
-		pTmpYMData = pYMWorkspace;
-		pNewYMData = pNewYMWorkspace;
-		*pNewYMData++ = *pTmpYMData++;
-		*pNewYMData++ = *pTmpYMData++;
-		*pNewYMData++ = *pTmpYMData++;
-		*pNewYMData++ = *pTmpYMData++;
-
-		/* Now copy over each stream */
-		for(Reg=0; Reg<NUM_PSG_SOUND_REGISTERS; Reg++)
-		{
-			/* Get pointer to source / destination */
-			pTmpYMStream = pTmpYMData + Reg;
-			pNewYMStream = pNewYMData + (Reg*nYMVBLS);
-
-			/* Copy recording VBLs worth */
-			for(Count=0; Count<nYMVBLS; Count++)
-			{
-				*pNewYMStream++ = *pTmpYMStream;
-				pTmpYMStream += NUM_PSG_SOUND_REGISTERS;
-			}
-		}
-
-		/* Delete old workspace and assign new */
-		free(pYMWorkspace);
-		pYMWorkspace = pNewYMWorkspace;
-
-		return TRUE;
-	}
-	else
-		return FALSE;
-}
